package fr.lip6.mcc.pnml.generators;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.dots.DotConstant;
import fr.lip6.move.pnml.symmetricnet.dots.DotsFactory;
import fr.lip6.move.pnml.symmetricnet.dots.hlapi.DotHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.NumberOfHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Tuple;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

@Preamble(
		author = "Lom",
		date = "23/03/2012",
		description = "Manual construction of the Petri net (ISO/IEC 15909-2) of Client/Server with Repetitions. \n",
		provider = "Alban Linard"
)
public final class ClientServerRepetitions extends RootGenerator implements PnmlGenerator {
	/**
	 * Root doc Class for the net
	 */
	private HLAPIRootClass csrDoc;

	public ClientServerRepetitions() {
		super();		
	}
	
	@Override
	public void generate2Pnml(String proprietaryFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Client Server Repetition instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Client Server Repetition instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done : instance " + instanceSize);
	}
	
	@Override
	protected void createModel(int instanceSize) {
		csrDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("csDoc"+ instanceSize, csrDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("clientServerRepetitions", PNTypeHLAPI.SYMNET, new NameHLAPI(
					"Client Server with Repetitions"), (PetriNetDocHLAPI) csrDoc);
			PageHLAPI defaultPage = new PageHLAPI("csrDefaultPage", net);
			/* Net declarations */
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class Client is 1..NCLIENTS (where NCLIENTS = instanceSize)
			CyclicEnumerationHLAPI clEnum = new CyclicEnumerationHLAPI();
			for (int i = 1; i <= instanceSize; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("cId" + String.valueOf(i), String.valueOf(i));
				clEnum.addElementsHLAPI(oneConstant);
			}
			NamedSortHLAPI clName = new NamedSortHLAPI("clclass", "Client", clEnum, declarations);
			// Class Server is 1..NSERVERS (where NSERVERS = sqrt(instanceSize))
			CyclicEnumerationHLAPI srEnum = new CyclicEnumerationHLAPI();
			for (int i = 1; i <= Math.sqrt(instanceSize) ; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("sId" + String.valueOf(i), String.valueOf(i));
				srEnum.addElementsHLAPI(oneConstant);
			}
			NamedSortHLAPI srName = new NamedSortHLAPI("srclass", "Server", srEnum, declarations);
			// Dot declaration for non-coloured annotations
			DotHLAPI dot = new DotHLAPI();
			NamedSortHLAPI dotName = new NamedSortHLAPI("dot", "Dot", dot, declarations);
			// Domain Computation is <Client,Server>
			ProductSortHLAPI csDomain = new ProductSortHLAPI();
			csDomain.addElementSortHLAPI(new UserSortHLAPI(clName));
			csDomain.addElementSortHLAPI(new UserSortHLAPI(srName));
			NamedSortHLAPI csDomainName = new NamedSortHLAPI("computation", "Computation", csDomain, declarations);
			// Var c in Client
			VariableDeclHLAPI cDecl = new VariableDeclHLAPI("varc", "c", new UserSortHLAPI(clName), declarations);
			// Var s in Server
			VariableDeclHLAPI sDecl = new VariableDeclHLAPI("vars", "s", new UserSortHLAPI(srName), declarations);	
			/* Places */
			// Place RequestBufferSlots; {dom: dot; mark: BUFFERSIZE'(dot), where BUFFERSIZE = instanceSize ^ 2}
			NumberConstant bufferSize = IntegersFactory.eINSTANCE.createNumberConstant();
			bufferSize.setValue((int) Math.pow(instanceSize, 2));
			bufferSize.setType(IntegersFactory.eINSTANCE.createPositive());
			DotConstant dotConstant = DotsFactory.eINSTANCE.createDotConstant();
			NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof0.getSubterm().add(bufferSize);
			nof0.getSubterm().add(dotConstant);
			PlaceHLAPI requestBufferSlots = new PlaceHLAPI("requestBufferSlots", new NameHLAPI("RequestBufferSlots"), null, new TypeHLAPI(null, "Dot",
					new UserSortHLAPI(dotName)), new HLMarkingHLAPI(null, bufferSize.getValue() + "'(dot)", new NumberOfHLAPI(nof0)),
					defaultPage);
			// Place Client_Sending {dom: Client}
			PlaceHLAPI clientSending = new PlaceHLAPI("clientSending", new NameHLAPI("Client_Sending"), null, new TypeHLAPI(null,
					"Client", new UserSortHLAPI(clName)), null, defaultPage);
			// Place RequestBuffer {dom: Client}
			PlaceHLAPI requestBuffer = new PlaceHLAPI("requestBuffer", new NameHLAPI("RequestBuffer"), null, new TypeHLAPI(null,
					"Client", new UserSortHLAPI(clName)), null, defaultPage);
			// Place Server_Waiting {dom: Server; mark: Server.all}
			PlaceHLAPI serverWaiting = new PlaceHLAPI("serverWaiting", new NameHLAPI("Server_Waiting"), null, new TypeHLAPI(null, "Server",
					new UserSortHLAPI(srName)),
					new HLMarkingHLAPI(null, "Server.all", new AllHLAPI(new UserSortHLAPI(srName))), defaultPage);
			// Place Client_Waiting {dom: client, mark: Client.all}
			PlaceHLAPI clientWaiting = new PlaceHLAPI("clientWaiting", new NameHLAPI("Client_Waiting"), null, new TypeHLAPI(null, "Client",
					new UserSortHLAPI(clName)),
					new HLMarkingHLAPI(null, "Client.all", new AllHLAPI(new UserSortHLAPI(clName))), defaultPage);
			// Place Server_Anwering {dom: Computation}
			PlaceHLAPI serverAnwering = new PlaceHLAPI("serverAnwering", new NameHLAPI("Server_Anwering"), null, new TypeHLAPI(null,
					"Computation", new UserSortHLAPI(csDomainName)), null, defaultPage);
			/* Transitions */
			// Transition Send_Request
			TransitionHLAPI sendRequest = new TransitionHLAPI("sendRequest", new NameHLAPI("Send_Request"), null,
					null, defaultPage);
			// Transition Read_Request
			TransitionHLAPI readRequest = new TransitionHLAPI("readRequest", new NameHLAPI("Read_Request"), null,
					null, defaultPage);
			// Transition Prepare_Request
			TransitionHLAPI prepareRequest = new TransitionHLAPI("prepareRequest", new NameHLAPI("Prepare_Request"), null,
					null, defaultPage);
			// Transition Lose_Request
			TransitionHLAPI loseRequest = new TransitionHLAPI("loseRequest", new NameHLAPI("Lose_Request"), null,
					null, defaultPage);
			// Transition Send_Answer
			TransitionHLAPI sendAnswer = new TransitionHLAPI("sendAnswer", new NameHLAPI("Send_Answer"), null,
					null, defaultPage);
			/* Arcs */
			// RequestBufferSlots --> Send_Request : 1'(dot)
			NumberConstant one1 = IntegersFactory.eINSTANCE.createNumberConstant();
			one1.setValue(1);
			one1.setType(IntegersFactory.eINSTANCE.createPositive());
			DotConstant dotConstant1 = DotsFactory.eINSTANCE.createDotConstant();
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one1);
			nof1.getSubterm().add(dotConstant1);
			HLAnnotation hlannot1 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot1.setStructure(nof1);
			hlannot1.setText("1'(dot)");
			ArcHLAPI requestBufferSlots2sendRequest = new ArcHLAPI("requestBufferSlots2sendRequest", new NameHLAPI("RequestBufferSlots2Send_Request"), requestBufferSlots, sendRequest, null,
					new HLAnnotationHLAPI(hlannot1), defaultPage);
			// ReadRequest --> RequestBufferSlots : 1'(dot)
			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			DotConstant dotConstant2 = DotsFactory.eINSTANCE.createDotConstant();
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one2);
			nof2.getSubterm().add(dotConstant2);
			HLAnnotation hlannot2 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot2.setStructure(nof2);
			hlannot2.setText("1'(dot)");
			ArcHLAPI readRequest2requestBufferSlots= new ArcHLAPI("readRequest2requestBufferSlots", new NameHLAPI("Read_Request2RequestBufferSlots"), readRequest, requestBufferSlots, null,
					new HLAnnotationHLAPI(hlannot2), defaultPage);
			// Client_Sending --> Send_Request : 1'(c)
			NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc1 = TermsFactory.eINSTANCE.createVariable();
			varc1.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one3);
			nof3.getSubterm().add(varc1);
			HLAnnotation hlannot3 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot3.setStructure(nof3);
			hlannot3.setText("1'(c)");
			ArcHLAPI clientSending2sendRequest = new ArcHLAPI("clientSending2sendRequest", new NameHLAPI("Client_Sending2Send_Request"), clientSending, sendRequest, null,
					new HLAnnotationHLAPI(hlannot3), defaultPage);
			// Send_Request --> Client_Sending : 1'(c)
			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc2 = TermsFactory.eINSTANCE.createVariable();
			varc2.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one4);
			nof4.getSubterm().add(varc2);
			HLAnnotation hlannot4 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot4.setStructure(nof4);
			hlannot4.setText("1'(c)");
			ArcHLAPI sendRequest2clientSending = new ArcHLAPI("sendRequest2clientSending", new NameHLAPI("Send_Request2Client_Sending"), sendRequest, clientSending, null,
					new HLAnnotationHLAPI(hlannot4), defaultPage);
			// Send_Request --> RequestBuffer : 1'(c)
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc3 = TermsFactory.eINSTANCE.createVariable();
			varc3.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(one5);
			nof5.getSubterm().add(varc3);
			HLAnnotation hlannot5 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot5.setStructure(nof5);
			hlannot5.setText("1'(c)");
			ArcHLAPI sendRequest2requestBuffer = new ArcHLAPI("sendRequest2requestBuffer", new NameHLAPI("Send_Request2RequestBuffer"), sendRequest, requestBuffer, null,
					new HLAnnotationHLAPI(hlannot5), defaultPage);
			// Request_Buffer --> Read_Request : 1'(c)
			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc4 = TermsFactory.eINSTANCE.createVariable();
			varc4.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(one6);
			nof6.getSubterm().add(varc4);
			HLAnnotation hlannot6 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot6.setStructure(nof6);
			hlannot6.setText("1'(c)");
			ArcHLAPI requestBuffer2readRequest = new ArcHLAPI("requestBuffer2readRequest", new NameHLAPI("RequestBuffer2Read_Request"), requestBuffer, readRequest,null,
					new HLAnnotationHLAPI(hlannot6), defaultPage);
			// Request_Buffer --> Lost_Request : 1'(c)
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc5 = TermsFactory.eINSTANCE.createVariable();
			varc5.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(one7);
			nof7.getSubterm().add(varc5);
			HLAnnotation hlannot7 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot7.setStructure(nof7);
			hlannot7.setText("1'(c)");
			ArcHLAPI requestBuffer2loseRequest = new ArcHLAPI("requestBuffer2loseRequest", new NameHLAPI("RequestBuffer2Lose_Request"), requestBuffer, loseRequest,null,
					new HLAnnotationHLAPI(hlannot7), defaultPage);
			// Server_Waiting --> Read_Request : 1'(s)
			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vars0 = TermsFactory.eINSTANCE.createVariable();
			vars0.setVariableDecl(sDecl.getContainedItem());
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one8);
			nof8.getSubterm().add(vars0);
			HLAnnotation hlannot8 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot8.setStructure(nof8);
			hlannot8.setText("1'(s)");
			ArcHLAPI serverWaiting2readRequest = new ArcHLAPI("serverWaiting2readRequest", new NameHLAPI("Server_Waiting2Read_Request"), serverWaiting, readRequest, null,
					new HLAnnotationHLAPI(hlannot8), defaultPage);
			// Read_Request --> Server_Anwering : 1'(c,s)
			NumberConstant one9 = IntegersFactory.eINSTANCE.createNumberConstant();
			one9.setValue(1);
			one9.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc6 = TermsFactory.eINSTANCE.createVariable();
			varc6.setVariableDecl(cDecl.getContainedItem());
			Variable vars1 = TermsFactory.eINSTANCE.createVariable();
			vars1.setVariableDecl(sDecl.getContainedItem());
			Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
			tuple0.getSubterm().add(varc6);
			tuple0.getSubterm().add(vars1);
			NumberOf nof9 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof9.getSubterm().add(one9);
			nof9.getSubterm().add(tuple0);
			HLAnnotation hlannot9 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot9.setStructure(nof9);
			hlannot9.setText("1'(c,s)");
			ArcHLAPI readRequest2ServerAnwering= new ArcHLAPI("readRequest2ServerAnwering", new NameHLAPI("Read_Request2Server_Anwering"), readRequest, serverAnwering,
					null, new HLAnnotationHLAPI(hlannot9), defaultPage);
			// Client_Sending --> Send_Answer : 1'(c)
			NumberConstant one10 = IntegersFactory.eINSTANCE.createNumberConstant();
			one10.setValue(1);
			one10.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc7 = TermsFactory.eINSTANCE.createVariable();
			varc7.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof10 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof10.getSubterm().add(one10);
			nof10.getSubterm().add(varc7);
			HLAnnotation hlannot11 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot11.setStructure(nof10);
			hlannot11.setText("1'(c)");
			ArcHLAPI clientSending2SendAnswer = new ArcHLAPI("clientSending2SendAnswer", new NameHLAPI("Client_Sending2Send_Answer"), clientSending, sendAnswer, null,
					new HLAnnotationHLAPI(hlannot11), defaultPage);
			// Send_Answer --> Server_Waiting : 1'(s)
			NumberConstant one11 = IntegersFactory.eINSTANCE.createNumberConstant();
			one11.setValue(1);
			one11.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vars2 = TermsFactory.eINSTANCE.createVariable();
			vars2.setVariableDecl(sDecl.getContainedItem());
			NumberOf nof11 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof11.getSubterm().add(one11);
			nof11.getSubterm().add(vars2);
			HLAnnotation hlannot12 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot12.setStructure(nof11);
			hlannot12.setText("1'(s)");
			ArcHLAPI sendAnswer2serverWaiting = new ArcHLAPI("sendAnswer2serverWaiting", new NameHLAPI("Send_Answer2Server_Waiting"), sendAnswer, serverWaiting, null,
					new HLAnnotationHLAPI(hlannot12), defaultPage);
			// Client_Waiting --> Prepare_Request : 1'(c)
			NumberConstant one12 = IntegersFactory.eINSTANCE.createNumberConstant();
			one12.setValue(1);
			one12.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc8 = TermsFactory.eINSTANCE.createVariable();
			varc8.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof12 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof12.getSubterm().add(one12);
			nof12.getSubterm().add(varc8);
			HLAnnotation hlannot13 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot13.setStructure(nof12);
			hlannot13.setText("1'(c)");
			ArcHLAPI clientWaiting2PrepareRequest = new ArcHLAPI("clientWaiting2PrepareRequest", new NameHLAPI("Client_Waiting2Prepare_Request"), clientWaiting, prepareRequest, null,
					new HLAnnotationHLAPI(hlannot13), defaultPage);
			// Prepare_Request --> Client_Sending : 1'(c)
			NumberConstant one13 = IntegersFactory.eINSTANCE.createNumberConstant();
			one13.setValue(1);
			one13.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc9 = TermsFactory.eINSTANCE.createVariable();
			varc9.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof13 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof13.getSubterm().add(one13);
			nof13.getSubterm().add(varc9);
			HLAnnotation hlannot14 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot14.setStructure(nof13);
			hlannot14.setText("1'(c)");
			ArcHLAPI prepareRequest2ClientSending = new ArcHLAPI("prepareRequest2ClientSending", new NameHLAPI("Prepare_Request2Client_Sending"), prepareRequest, clientSending, null,
					new HLAnnotationHLAPI(hlannot14), defaultPage);
			// Send_Anwering --> Send_Answer : 1'(c,s)
			NumberConstant one14 = IntegersFactory.eINSTANCE.createNumberConstant();
			one14.setValue(1);
			one14.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc10 = TermsFactory.eINSTANCE.createVariable();
			varc10.setVariableDecl(cDecl.getContainedItem());
			Variable vars3 = TermsFactory.eINSTANCE.createVariable();
			vars3.setVariableDecl(sDecl.getContainedItem());
			Tuple tuple1 = TermsFactory.eINSTANCE.createTuple();
			tuple1.getSubterm().add(varc10);
			tuple1.getSubterm().add(vars3);
			NumberOf nof14 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof14.getSubterm().add(one14);
			nof14.getSubterm().add(tuple1);
			HLAnnotation hlannot15 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot15.setStructure(nof14);
			hlannot15.setText("1'(c,s)");
		ArcHLAPI serverAnwering2SendAnswer = new ArcHLAPI("serverAnwering2SendAnswer", new NameHLAPI("Server_Anwering2Send_Answer"), serverAnwering, sendAnswer,
					null, new HLAnnotationHLAPI(hlannot15), defaultPage);
			// Send_Answer --> Client_Waiting : 1'(c)
			NumberConstant one15 = IntegersFactory.eINSTANCE.createNumberConstant();
			one15.setValue(1);
			one15.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varc11 = TermsFactory.eINSTANCE.createVariable();
			varc11.setVariableDecl(cDecl.getContainedItem());
			NumberOf nof15 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof15.getSubterm().add(one15);
			nof15.getSubterm().add(varc11);
			HLAnnotation hlannot16 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot16.setStructure(nof15);
			hlannot16.setText("1'(c)");
			ArcHLAPI sendAnswer2ClientWaiting = new ArcHLAPI("sendAnswer2ClientWaiting", new NameHLAPI("Send_Answer2Client_Waiting"), sendAnswer, clientWaiting, null,
					new HLAnnotationHLAPI(hlannot16), defaultPage);

		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}
	}
}
