package fr.lip6.mcc.pnml.generators;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.booleans.BooleansFactory;
import fr.lip6.move.pnml.symmetricnet.booleans.Inequality;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.Condition;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ConditionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.Add;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.Subtract;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Tuple;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

@Preamble(
		author = "Lom",
		date = "25/03/2012",
		description = "Manual construction of the Petri net (ISO/IEC 15909-2) of Dynamic Philosophers. \n",
		provider = "Alban Linard"
)
public class DynPhilo extends RootGenerator implements PnmlGenerator {
	/**
	 * Root doc Class for the net
	 */
	private HLAPIRootClass phiDoc;

	public DynPhilo() {
		super();
	}

	@Override
	public void generate2Pnml(String proprietaryFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Dynamic Philosophers instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Dynamic Philosophers instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done : Dynamic Philosophers instance " + instanceSize);
	}

	@Override
	protected void createModel(int instanceSize) {
		phiDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("phiDoc"+ instanceSize, phiDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("dynamicPhilo", PNTypeHLAPI.SYMNET, new NameHLAPI(
					"Dynamic Philosophers"), (PetriNetDocHLAPI) phiDoc);
			PageHLAPI defaultPage = new PageHLAPI("philodynDefaultPage", net);
			/* Net declarations */
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class Philosophers is 1..N (where N = instanceSize)
			CyclicEnumerationHLAPI phiEnum = new CyclicEnumerationHLAPI();
			for (int i = 1; i <= instanceSize; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("philoId" + String.valueOf(i), String.valueOf(i));
				phiEnum.addElementsHLAPI(oneConstant);
			}
			NamedSortHLAPI phiName = new NamedSortHLAPI("philo", "Philosopher", phiEnum, declarations);
			// Domain Neighbourhood is <Philosophers,Philosophers>
			ProductSortHLAPI philoDomain = new ProductSortHLAPI();
			philoDomain.addElementSortHLAPI(new UserSortHLAPI(phiName));
			philoDomain.addElementSortHLAPI(new UserSortHLAPI(phiName));
			NamedSortHLAPI philoDomainName = new NamedSortHLAPI("philoD", "Neighbourhood", philoDomain, declarations);
			// Var p, q, l, r in Philosophers
			VariableDeclHLAPI pDecl = new VariableDeclHLAPI("varp", "p", new UserSortHLAPI(phiName), declarations);
			VariableDeclHLAPI qDecl = new VariableDeclHLAPI("varq", "q", new UserSortHLAPI(phiName), declarations);
			VariableDeclHLAPI lDecl = new VariableDeclHLAPI("varl", "l", new UserSortHLAPI(phiName), declarations);
			VariableDeclHLAPI rDecl = new VariableDeclHLAPI("varr", "r", new UserSortHLAPI(phiName), declarations);
			/* Places */
			// Place Outside {dom: Philosopher; mark: Philosophers.all}
			PlaceHLAPI outside = new PlaceHLAPI("outside", new NameHLAPI("Outside"), null, new TypeHLAPI(null, "Philosopher",
					new UserSortHLAPI(phiName)),
					new HLMarkingHLAPI(null, "Philosopher.all", new AllHLAPI(new UserSortHLAPI(phiName))), defaultPage);
			// Place Neighbourhood {dom: Neighbourhood}
			PlaceHLAPI neighbourhood = new PlaceHLAPI("neighbourhood", new NameHLAPI("Neighbourhood"), null, new TypeHLAPI(null,
					"Neighbourhood", new UserSortHLAPI(philoDomainName)), null, defaultPage);
			// Place Think {dom: Philosopher}
			PlaceHLAPI think = new PlaceHLAPI("think", new NameHLAPI("Think"), null, new TypeHLAPI(null,
					"Philosopher", new UserSortHLAPI(phiName)), null, defaultPage);
			// Place Fork {dom: Philosopher}
			PlaceHLAPI forks = new PlaceHLAPI("forks", new NameHLAPI("Forks"), null, new TypeHLAPI(null,
					"Philosopher", new UserSortHLAPI(phiName)), null, defaultPage);
			// Place WaitLeft {dom: Philosopher}
			PlaceHLAPI waitLeft = new PlaceHLAPI("waitleft", new NameHLAPI("WaitLeft"), null, new TypeHLAPI(null,
					"Philosopher", new UserSortHLAPI(phiName)), null, defaultPage);
			// Place WaitRight {dom: Philosopher}
			PlaceHLAPI waitRight = new PlaceHLAPI("waitright", new NameHLAPI("WaitRight"), null, new TypeHLAPI(null,
					"Philosopher", new UserSortHLAPI(phiName)), null, defaultPage);
			// Place HasLeft {dom: Philosopher}
			PlaceHLAPI hasLeft = new PlaceHLAPI("hasleft", new NameHLAPI("HasLeft"), null, new TypeHLAPI(null,
					"Philosopher", new UserSortHLAPI(phiName)), null, defaultPage);
			// Place HasRight {dom: Philosopher}
			PlaceHLAPI hasRight = new PlaceHLAPI("hasright", new NameHLAPI("HasRight"), null, new TypeHLAPI(null,
					"Philosopher", new UserSortHLAPI(phiName)), null, defaultPage);
			/* Transitions */
			// Initialize {p!=q}
			Variable varp0 = TermsFactory.eINSTANCE.createVariable();
			varp0.setVariableDecl(pDecl.getContainedItem());
			Variable varq0 = TermsFactory.eINSTANCE.createVariable();
			varq0.setVariableDecl(qDecl.getContainedItem());
			Inequality ineq0 = BooleansFactory.eINSTANCE.createInequality();
			ineq0.getSubterm().add(varp0);
			ineq0.getSubterm().add(varq0);
			Condition cond0 = HlcorestructureFactory.eINSTANCE.createCondition();
			cond0.setStructure(ineq0);
			cond0.setText("[p!=q]");
			TransitionHLAPI initialize = new TransitionHLAPI("initialize", new NameHLAPI("Initialize"), null,
					new ConditionHLAPI(cond0), defaultPage);
			// Transition Join
			TransitionHLAPI join = new TransitionHLAPI("join", new NameHLAPI("Join"), null,
					null, defaultPage);
			// Transition Leave
			TransitionHLAPI leave = new TransitionHLAPI("leave", new NameHLAPI("Leave"), null,
					null, defaultPage);
			// Transition SearchForks
			TransitionHLAPI searchForks = new TransitionHLAPI("searchforks", new NameHLAPI("SearchForks"), null,
					null, defaultPage);
			// Transition TakeLeft
			TransitionHLAPI takeLeft = new TransitionHLAPI("takeleft", new NameHLAPI("TakeLeft"), null,
					null, defaultPage);
			// Transition TakeRight
			TransitionHLAPI takeRight = new TransitionHLAPI("takeright", new NameHLAPI("TakeRight"), null,
					null, defaultPage);
			// Transition Eat
			TransitionHLAPI eat = new TransitionHLAPI("eat", new NameHLAPI("Eat"), null,
					null, defaultPage);
			/* Arcs */
			// Outside --> Join : 1'(p)
			NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
			one0.setValue(1);
			one0.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp1 = TermsFactory.eINSTANCE.createVariable();
			varp1.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof0.getSubterm().add(one0);
			nof0.getSubterm().add(varp1);
			HLAnnotation hlannot0 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot0.setStructure(nof0);
			hlannot0.setText("1'(p)");
			ArcHLAPI outside2join = new ArcHLAPI("outside2join", new NameHLAPI("Outside2Join"), outside, join, null,
					new HLAnnotationHLAPI(hlannot0), defaultPage);
			// Leave --> Outside : 1'(p)
			NumberConstant one1 = IntegersFactory.eINSTANCE.createNumberConstant();
			one1.setValue(1);
			one1.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp2 = TermsFactory.eINSTANCE.createVariable();
			varp2.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one1);
			nof1.getSubterm().add(varp2);
			HLAnnotation hlannot1 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot1.setStructure(nof1);
			hlannot1.setText("1'(p)");
			ArcHLAPI leave2outside = new ArcHLAPI("leave2outside", new NameHLAPI("Leave2Outside"), leave, outside, null,
					new HLAnnotationHLAPI(hlannot1), defaultPage);
			// Outside --> Initialize : Philosophers.all
			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			AllHLAPI all0 = new AllHLAPI(new UserSortHLAPI(phiName));
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one2);
			nof2.getSubterm().add(all0.getContainedItem());
			HLAnnotation hlannot2 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot2.setStructure(nof2);
			hlannot2.setText("1'(Philosophers.all)");
			ArcHLAPI outside2initialize = new ArcHLAPI("outside2initialize", new NameHLAPI("Outside2Initialize"), outside, initialize, null,
					new HLAnnotationHLAPI(hlannot2), defaultPage);
			//Initialize --> Outside : Philosophers.all - 1'(p) - 1'(q)
			NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());
			AllHLAPI all1 = new AllHLAPI(new UserSortHLAPI(phiName));
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one3);
			nof3.getSubterm().add(all1.getContainedItem());
			
			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp3 = TermsFactory.eINSTANCE.createVariable();
			varp3.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one4);
			nof4.getSubterm().add(varp3);
			
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varq1 = TermsFactory.eINSTANCE.createVariable();
			varq1.setVariableDecl(qDecl.getContainedItem());
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(one5);
			nof5.getSubterm().add(varq1);
			
			Subtract sub0 = MultisetsFactory.eINSTANCE.createSubtract();
			sub0.getSubterm().add(nof3);
			sub0.getSubterm().add(nof4);
			Subtract sub1 = MultisetsFactory.eINSTANCE.createSubtract();
			sub1.getSubterm().add(sub0);
			sub1.getSubterm().add(nof5);
			
			HLAnnotation hlannot3 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot3.setStructure(sub1);
			hlannot3.setText("1'(Philosophers.all) - 1'(p) - 1'(q)");
			ArcHLAPI initialize2outside = new ArcHLAPI("initialize2outside", new NameHLAPI("Initialize2Outside"), initialize, outside, null,
					new HLAnnotationHLAPI(hlannot3), defaultPage);
			// Neighbourhood --> Join : 1'(l,r)
			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varl0 = TermsFactory.eINSTANCE.createVariable();
			varl0.setVariableDecl(lDecl.getContainedItem());
			Variable varr0 = TermsFactory.eINSTANCE.createVariable();
			varr0.setVariableDecl(rDecl.getContainedItem());
			Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
			tuple0.getSubterm().add(varl0);
			tuple0.getSubterm().add(varr0);
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(one6);
			nof6.getSubterm().add(tuple0);
			HLAnnotation hlannot4 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot4.setStructure(nof6);
			hlannot4.setText("1'(l,r)");
			ArcHLAPI neighbourhood2join = new ArcHLAPI("neighbourhood2join", new NameHLAPI("Neighbourhood2Join"), neighbourhood, join,
					null, new HLAnnotationHLAPI(hlannot4), defaultPage);
			// Leave --> Neighbourhood : 1'(l,r)
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varl1 = TermsFactory.eINSTANCE.createVariable();
			varl1.setVariableDecl(lDecl.getContainedItem());
			Variable varr1 = TermsFactory.eINSTANCE.createVariable();
			varr1.setVariableDecl(rDecl.getContainedItem());
			Tuple tuple1 = TermsFactory.eINSTANCE.createTuple();
			tuple1.getSubterm().add(varl1);
			tuple1.getSubterm().add(varr1);
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(one7);
			nof7.getSubterm().add(tuple1);
			HLAnnotation hlannot5 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot5.setStructure(nof7);
			hlannot5.setText("1'(l,r)");
			ArcHLAPI leave2neighbourhood = new ArcHLAPI("leave2neighbourhood", new NameHLAPI("Leave2Neighbourhood"), leave, neighbourhood,
					null, new HLAnnotationHLAPI(hlannot5), defaultPage);
			// Neighbourhood --> Leave : 1'(l,p) + 1'(p,r)
			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varl2 = TermsFactory.eINSTANCE.createVariable();
			varl2.setVariableDecl(lDecl.getContainedItem());
			Variable varp4 = TermsFactory.eINSTANCE.createVariable();
			varp4.setVariableDecl(pDecl.getContainedItem());
			Tuple tuple2 = TermsFactory.eINSTANCE.createTuple();
			tuple2.getSubterm().add(varl2);
			tuple2.getSubterm().add(varp4);
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one8);
			nof8.getSubterm().add(tuple2);

			NumberConstant one9 = IntegersFactory.eINSTANCE.createNumberConstant();
			one9.setValue(1);
			one9.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp5 = TermsFactory.eINSTANCE.createVariable();
			varp5.setVariableDecl(pDecl.getContainedItem());
			Variable varr2 = TermsFactory.eINSTANCE.createVariable();
			varr2.setVariableDecl(rDecl.getContainedItem());
			Tuple tuple3 = TermsFactory.eINSTANCE.createTuple();
			tuple3.getSubterm().add(varp5);
			tuple3.getSubterm().add(varr2);
			NumberOf nof9 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof9.getSubterm().add(one9);
			nof9.getSubterm().add(tuple3);

			Add add0 = MultisetsFactory.eINSTANCE.createAdd();
			add0.getSubterm().add(nof8);
			add0.getSubterm().add(nof9);

			HLAnnotation hlannot6 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot6.setStructure(add0);
			hlannot6.setText("1'(l,p) + 1'(p,r)");
			ArcHLAPI neighbourhood2leave = new ArcHLAPI("neighbourhood2leave", new NameHLAPI("Neighbourhood2Leave"), neighbourhood,
					leave, null, new HLAnnotationHLAPI(hlannot6), defaultPage);
			// Join --> Neighbourhood : 1'(l,p) + 1'(p,r)
			NumberConstant one10 = IntegersFactory.eINSTANCE.createNumberConstant();
			one10.setValue(1);
			one10.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varl3 = TermsFactory.eINSTANCE.createVariable();
			varl3.setVariableDecl(lDecl.getContainedItem());
			Variable varp6 = TermsFactory.eINSTANCE.createVariable();
			varp6.setVariableDecl(pDecl.getContainedItem());
			Tuple tuple4 = TermsFactory.eINSTANCE.createTuple();
			tuple4.getSubterm().add(varl3);
			tuple4.getSubterm().add(varp6);
			NumberOf nof10 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof10.getSubterm().add(one10);
			nof10.getSubterm().add(tuple4);

			NumberConstant one11 = IntegersFactory.eINSTANCE.createNumberConstant();
			one11.setValue(1);
			one11.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp7 = TermsFactory.eINSTANCE.createVariable();
			varp7.setVariableDecl(pDecl.getContainedItem());
			Variable varr3 = TermsFactory.eINSTANCE.createVariable();
			varr3.setVariableDecl(rDecl.getContainedItem());
			Tuple tuple5 = TermsFactory.eINSTANCE.createTuple();
			tuple5.getSubterm().add(varp7);
			tuple5.getSubterm().add(varr3);
			NumberOf nof11 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof11.getSubterm().add(one11);
			nof11.getSubterm().add(tuple5);

			Add add1 = MultisetsFactory.eINSTANCE.createAdd();
			add1.getSubterm().add(nof10);
			add1.getSubterm().add(nof11);

			HLAnnotation hlannot7 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot7.setStructure(add1);
			hlannot7.setText("1'(l,p) + 1'(p,r)");
			ArcHLAPI join2neighbourhood = new ArcHLAPI("join2neighbourhood", new NameHLAPI("Join2Neighbourhood"), join, neighbourhood,
					null, new HLAnnotationHLAPI(hlannot7), defaultPage);
			// Initialize --> Neighbourhood : 1'(p,q) + 1'(q,p)
			NumberConstant one12 = IntegersFactory.eINSTANCE.createNumberConstant();
			one12.setValue(1);
			one12.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp8 = TermsFactory.eINSTANCE.createVariable();
			varp8.setVariableDecl(pDecl.getContainedItem());
			Variable varq2 = TermsFactory.eINSTANCE.createVariable();
			varq2.setVariableDecl(qDecl.getContainedItem());
			Tuple tuple6 = TermsFactory.eINSTANCE.createTuple();
			tuple6.getSubterm().add(varp8);
			tuple6.getSubterm().add(varq2);
			NumberOf nof12 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof12.getSubterm().add(one12);
			nof12.getSubterm().add(tuple6);

			NumberConstant one13 = IntegersFactory.eINSTANCE.createNumberConstant();
			one13.setValue(1);
			one13.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varq3 = TermsFactory.eINSTANCE.createVariable();
			varq3.setVariableDecl(qDecl.getContainedItem());
			Variable varp9 = TermsFactory.eINSTANCE.createVariable();
			varp9.setVariableDecl(pDecl.getContainedItem());
			Tuple tuple7 = TermsFactory.eINSTANCE.createTuple();
			tuple7.getSubterm().add(varq3);
			tuple7.getSubterm().add(varp9);
			NumberOf nof13 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof13.getSubterm().add(one13);
			nof13.getSubterm().add(tuple7);

			Add add2 = MultisetsFactory.eINSTANCE.createAdd();
			add2.getSubterm().add(nof12);
			add2.getSubterm().add(nof13);

			HLAnnotation hlannot8 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot8.setStructure(add2);
			hlannot8.setText("1'(p,q) + 1'(q,p)");
			ArcHLAPI initialize2neighbourhood = new ArcHLAPI("initialize2neighbourhood", new NameHLAPI("Initialize2Neighbourhood"), initialize, neighbourhood,
					null, new HLAnnotationHLAPI(hlannot8), defaultPage);
			// TakeRight --> Neighbourhood : 1'(p,q)
			NumberConstant one14 = IntegersFactory.eINSTANCE.createNumberConstant();
			one14.setValue(1);
			one14.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp10 = TermsFactory.eINSTANCE.createVariable();
			varp10.setVariableDecl(pDecl.getContainedItem());
			Variable varq4 = TermsFactory.eINSTANCE.createVariable();
			varq4.setVariableDecl(qDecl.getContainedItem());
			Tuple tuple8 = TermsFactory.eINSTANCE.createTuple();
			tuple8.getSubterm().add(varp10);
			tuple8.getSubterm().add(varq4);
			NumberOf nof14 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof14.getSubterm().add(one14);
			nof14.getSubterm().add(tuple8);
			HLAnnotation hlannot9 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot9.setStructure(nof14);
			hlannot9.setText("1'(p,q)");
			ArcHLAPI takeright2neighbourhood = new ArcHLAPI("takeright2neighbourhood", new NameHLAPI("TakeRight2Neighbourhood"), takeRight, neighbourhood,
					null, new HLAnnotationHLAPI(hlannot9), defaultPage);
			//  Neighbourhood --> TakeRight ->: 1'(p,q)
			NumberConstant one15 = IntegersFactory.eINSTANCE.createNumberConstant();
			one15.setValue(1);
			one15.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp11 = TermsFactory.eINSTANCE.createVariable();
			varp11.setVariableDecl(pDecl.getContainedItem());
			Variable varq5 = TermsFactory.eINSTANCE.createVariable();
			varq5.setVariableDecl(qDecl.getContainedItem());
			Tuple tuple9 = TermsFactory.eINSTANCE.createTuple();
			tuple9.getSubterm().add(varp11);
			tuple9.getSubterm().add(varq5);
			NumberOf nof15 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof15.getSubterm().add(one15);
			nof15.getSubterm().add(tuple9);
			HLAnnotation hlannot10 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot10.setStructure(nof15);
			hlannot10.setText("1'(p,q)");
			ArcHLAPI neighbourhood2takeright = new ArcHLAPI("neighbourhood2takeright", new NameHLAPI("Neighbourhood2TakeRight"), neighbourhood, takeRight,
					null, new HLAnnotationHLAPI(hlannot10), defaultPage);
			// Neighbourhood --> Eat : 1'(p,q)
			NumberConstant one16 = IntegersFactory.eINSTANCE.createNumberConstant();
			one16.setValue(1);
			one16.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp12 = TermsFactory.eINSTANCE.createVariable();
			varp12.setVariableDecl(pDecl.getContainedItem());
			Variable varq6 = TermsFactory.eINSTANCE.createVariable();
			varq6.setVariableDecl(qDecl.getContainedItem());
			Tuple tuple10 = TermsFactory.eINSTANCE.createTuple();
			tuple10.getSubterm().add(varp12);
			tuple10.getSubterm().add(varq6);
			NumberOf nof16 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof16.getSubterm().add(one16);
			nof16.getSubterm().add(tuple10);
			HLAnnotation hlannot11 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot11.setStructure(nof16);
			hlannot11.setText("1'(p,q)");
			ArcHLAPI neighbourhood2eat = new ArcHLAPI("neighbourhood2eat", new NameHLAPI("Neighbourhood2Eat"), neighbourhood, eat,
					null, new HLAnnotationHLAPI(hlannot11), defaultPage);
			// Eat --> Neighbourhood : 1'(p,q)
			NumberConstant one17 = IntegersFactory.eINSTANCE.createNumberConstant();
			one17.setValue(1);
			one17.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp13 = TermsFactory.eINSTANCE.createVariable();
			varp13.setVariableDecl(pDecl.getContainedItem());
			Variable varq7 = TermsFactory.eINSTANCE.createVariable();
			varq7.setVariableDecl(qDecl.getContainedItem());
			Tuple tuple11 = TermsFactory.eINSTANCE.createTuple();
			tuple11.getSubterm().add(varp13);
			tuple11.getSubterm().add(varq7);
			NumberOf nof17 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof17.getSubterm().add(one17);
			nof17.getSubterm().add(tuple11);
			HLAnnotation hlannot12 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot12.setStructure(nof17);
			hlannot12.setText("1'(p,q)");
			ArcHLAPI eat2neighbourhood = new ArcHLAPI("eat2neighbourhood", new NameHLAPI("Eat2Neighbourhood"), eat, neighbourhood,
					null, new HLAnnotationHLAPI(hlannot12), defaultPage);
			// Join --> Think : 1'(p)
			NumberConstant one18 = IntegersFactory.eINSTANCE.createNumberConstant();
			one18.setValue(1);
			one18.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp14 = TermsFactory.eINSTANCE.createVariable();
			varp14.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof18 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof18.getSubterm().add(one18);
			nof18.getSubterm().add(varp14);
			HLAnnotation hlannot13 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot13.setStructure(nof18);
			hlannot13.setText("1'(p)");
			ArcHLAPI join2think = new ArcHLAPI("join2think", new NameHLAPI("Join2Think"), join, think, null,
					new HLAnnotationHLAPI(hlannot13), defaultPage);
			// Think --> Leave : 1'(p)
			NumberConstant one19 = IntegersFactory.eINSTANCE.createNumberConstant();
			one19.setValue(1);
			one19.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp15 = TermsFactory.eINSTANCE.createVariable();
			varp15.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof19 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof19.getSubterm().add(one19);
			nof19.getSubterm().add(varp15);
			HLAnnotation hlannot14 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot14.setStructure(nof19);
			hlannot14.setText("1'(p)");
			ArcHLAPI think2leave = new ArcHLAPI("think2leave", new NameHLAPI("Think2Leave"), think, leave, null,
					new HLAnnotationHLAPI(hlannot14), defaultPage);
			// Think --> SearchForks : 1'(p)
			NumberConstant one20 = IntegersFactory.eINSTANCE.createNumberConstant();
			one20.setValue(1);
			one20.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp16 = TermsFactory.eINSTANCE.createVariable();
			varp16.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof20 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof20.getSubterm().add(one20);
			nof20.getSubterm().add(varp16);
			HLAnnotation hlannot15 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot15.setStructure(nof20);
			hlannot15.setText("1'(p)");
			ArcHLAPI think2searchforks = new ArcHLAPI("think2searchforks", new NameHLAPI("Think2SearchForks"), think, searchForks, null,
					new HLAnnotationHLAPI(hlannot15), defaultPage);
			// Eat --> Think : 1'(p)
			NumberConstant one21 = IntegersFactory.eINSTANCE.createNumberConstant();
			one21.setValue(1);
			one21.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp17 = TermsFactory.eINSTANCE.createVariable();
			varp17.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof21 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof21.getSubterm().add(one21);
			nof21.getSubterm().add(varp17);
			HLAnnotation hlannot16 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot16.setStructure(nof21);
			hlannot16.setText("1'(p)");
			ArcHLAPI eat2think = new ArcHLAPI("eat2think", new NameHLAPI("Eat2Think"), eat, think, null,
					new HLAnnotationHLAPI(hlannot16), defaultPage);
			// Initialize --> Think : 1'(p) + 1'(q)
			NumberConstant one22 = IntegersFactory.eINSTANCE.createNumberConstant();
			one22.setValue(1);
			one22.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp18 = TermsFactory.eINSTANCE.createVariable();
			varp18.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof22 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof22.getSubterm().add(one22);
			nof22.getSubterm().add(varp18);
			NumberConstant one23 = IntegersFactory.eINSTANCE.createNumberConstant();
			one23.setValue(1);
			one23.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varq8 = TermsFactory.eINSTANCE.createVariable();
			varq8.setVariableDecl(qDecl.getContainedItem());
			NumberOf nof23 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof23.getSubterm().add(one23);
			nof23.getSubterm().add(varq8);
			Add add3 = MultisetsFactory.eINSTANCE.createAdd();
			add3.getSubterm().add(nof22);
			add3.getSubterm().add(nof23);
			HLAnnotation hlannot17 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot17.setStructure(add3);
			hlannot17.setText("1'(p) + 1'(q)");
			ArcHLAPI initialize2think = new ArcHLAPI("initialize2think", new NameHLAPI("Initialize2Think"), initialize, think, null,
					new HLAnnotationHLAPI(hlannot17), defaultPage);
			// SearchForks --> WaitLeft : 1'(p)
			NumberConstant one24 = IntegersFactory.eINSTANCE.createNumberConstant();
			one24.setValue(1);
			one24.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp19 = TermsFactory.eINSTANCE.createVariable();
			varp19.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof24 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof24.getSubterm().add(one24);
			nof24.getSubterm().add(varp19);
			HLAnnotation hlannot18 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot18.setStructure(nof24);
			hlannot18.setText("1'(p)");
			ArcHLAPI searchforks2waitleft = new ArcHLAPI("searchforks2waitleft", new NameHLAPI("SearchForks2WaitLeft"), searchForks, waitLeft, null,
					new HLAnnotationHLAPI(hlannot18), defaultPage);
			// WaitLeft --> TakeLeft : 1'(p)
			NumberConstant one25 = IntegersFactory.eINSTANCE.createNumberConstant();
			one25.setValue(1);
			one25.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp20 = TermsFactory.eINSTANCE.createVariable();
			varp20.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof25 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof25.getSubterm().add(one25);
			nof25.getSubterm().add(varp20);
			HLAnnotation hlannot19 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot19.setStructure(nof25);
			hlannot19.setText("1'(p)");
			ArcHLAPI waitleft2takeleft = new ArcHLAPI("waitleft2takeleft", new NameHLAPI("WaitLeft2TakeLeft"), waitLeft, takeLeft, null,
					new HLAnnotationHLAPI(hlannot19), defaultPage);
			// WaitRight --> TakeRight : 1'(p)
			NumberConstant one26 = IntegersFactory.eINSTANCE.createNumberConstant();
			one26.setValue(1);
			one26.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp21 = TermsFactory.eINSTANCE.createVariable();
			varp21.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof26 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof26.getSubterm().add(one26);
			nof26.getSubterm().add(varp21);
			HLAnnotation hlannot20 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot20.setStructure(nof26);
			hlannot20.setText("1'(p)");
			ArcHLAPI waitright2takeright = new ArcHLAPI("waitright2takeright", new NameHLAPI("WaitRight2TakeRight"), waitRight, takeRight, null,
					new HLAnnotationHLAPI(hlannot20), defaultPage);
			// SearchForks --> WaitRight : 1'(p)
			NumberConstant one27 = IntegersFactory.eINSTANCE.createNumberConstant();
			one27.setValue(1);
			one27.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp22 = TermsFactory.eINSTANCE.createVariable();
			varp22.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof27 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof27.getSubterm().add(one27);
			nof27.getSubterm().add(varp22);
			HLAnnotation hlannot21 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot21.setStructure(nof27);
			hlannot21.setText("1'(p)");
			ArcHLAPI searchforks2waitright = new ArcHLAPI("searchforks2waitright", new NameHLAPI("SearchForks2WaitRight"), searchForks, waitRight, null,
					new HLAnnotationHLAPI(hlannot21), defaultPage);
			// Forks --> TakeLeft : 1'(p)
			NumberConstant one28 = IntegersFactory.eINSTANCE.createNumberConstant();
			one28.setValue(1);
			one28.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp23 = TermsFactory.eINSTANCE.createVariable();
			varp23.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof28 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof28.getSubterm().add(one28);
			nof28.getSubterm().add(varp23);
			HLAnnotation hlannot22 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot22.setStructure(nof28);
			hlannot22.setText("1'(p)");
			ArcHLAPI forks2takeleft = new ArcHLAPI("forks2takeleft", new NameHLAPI("Forks2TakeLeft"), forks, takeLeft, null,
					new HLAnnotationHLAPI(hlannot22), defaultPage);
			// Forks --> Leave : 1'(p)
			NumberConstant one29 = IntegersFactory.eINSTANCE.createNumberConstant();
			one29.setValue(1);
			one29.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp24 = TermsFactory.eINSTANCE.createVariable();
			varp24.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof29 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof29.getSubterm().add(one29);
			nof29.getSubterm().add(varp24);
			HLAnnotation hlannot23 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot23.setStructure(nof29);
			hlannot23.setText("1'(p)");
			ArcHLAPI forks2leave = new ArcHLAPI("forks2leave", new NameHLAPI("Forks2Leave"), forks, leave, null,
					new HLAnnotationHLAPI(hlannot23), defaultPage);
			// Forks --> TakeRight : 1'(q)
			NumberConstant one30 = IntegersFactory.eINSTANCE.createNumberConstant();
			one30.setValue(1);
			one30.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varq9 = TermsFactory.eINSTANCE.createVariable();
			varq9.setVariableDecl(qDecl.getContainedItem());
			NumberOf nof30 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof30.getSubterm().add(one30);
			nof30.getSubterm().add(varq9);
			HLAnnotation hlannot24 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot24.setStructure(nof30);
			hlannot24.setText("1'(q)");
			ArcHLAPI forks2takeright = new ArcHLAPI("forks2takeright", new NameHLAPI("Forks2TakeRight"), forks, takeRight, null,
					new HLAnnotationHLAPI(hlannot24), defaultPage);
			// Eat --> Forks : 1'(p) + 1'(q)
			NumberConstant one31 = IntegersFactory.eINSTANCE.createNumberConstant();
			one31.setValue(1);
			one31.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp25 = TermsFactory.eINSTANCE.createVariable();
			varp25.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof31 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof31.getSubterm().add(one31);
			nof31.getSubterm().add(varp25);
			NumberConstant one32 = IntegersFactory.eINSTANCE.createNumberConstant();
			one32.setValue(1);
			one32.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varq10 = TermsFactory.eINSTANCE.createVariable();
			varq10.setVariableDecl(qDecl.getContainedItem());
			NumberOf nof32 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof32.getSubterm().add(one32);
			nof32.getSubterm().add(varq10);
			Add add4 = MultisetsFactory.eINSTANCE.createAdd();
			add4.getSubterm().add(nof31);
			add4.getSubterm().add(nof32);
			HLAnnotation hlannot25 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot25.setStructure(add4);
			hlannot25.setText("1'(p) + 1'(q)");
			ArcHLAPI eat2forks = new ArcHLAPI("eat2forks", new NameHLAPI("Eat2Forks"), eat, forks, null,
					new HLAnnotationHLAPI(hlannot25), defaultPage);
			// Initialize --> Forks : 1'(p) + 1'(q)
			NumberConstant one33 = IntegersFactory.eINSTANCE.createNumberConstant();
			one33.setValue(1);
			one33.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp26 = TermsFactory.eINSTANCE.createVariable();
			varp26.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof33 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof33.getSubterm().add(one33);
			nof33.getSubterm().add(varp26);
			NumberConstant one34 = IntegersFactory.eINSTANCE.createNumberConstant();
			one34.setValue(1);
			one34.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varq11 = TermsFactory.eINSTANCE.createVariable();
			varq11.setVariableDecl(qDecl.getContainedItem());
			NumberOf nof34 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof34.getSubterm().add(one34);
			nof34.getSubterm().add(varq11);
			Add add5 = MultisetsFactory.eINSTANCE.createAdd();
			add5.getSubterm().add(nof33);
			add5.getSubterm().add(nof34);
			HLAnnotation hlannot26 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot26.setStructure(add5);
			hlannot26.setText("1'(p) + 1'(q)");
			ArcHLAPI initialize2forks = new ArcHLAPI("initialize2forks", new NameHLAPI("Initialize2Forks"), initialize, forks, null,
					new HLAnnotationHLAPI(hlannot26), defaultPage);
			// Join --> Forks : 1'(l) + 1'(r)
			NumberConstant one35 = IntegersFactory.eINSTANCE.createNumberConstant();
			one35.setValue(1);
			one35.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varl4 = TermsFactory.eINSTANCE.createVariable();
			varl4.setVariableDecl(lDecl.getContainedItem());
			NumberOf nof35 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof35.getSubterm().add(one35);
			nof35.getSubterm().add(varl4);
			NumberConstant one36 = IntegersFactory.eINSTANCE.createNumberConstant();
			one36.setValue(1);
			one36.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr4 = TermsFactory.eINSTANCE.createVariable();
			varr4.setVariableDecl(rDecl.getContainedItem());
			NumberOf nof36 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof36.getSubterm().add(one36);
			nof36.getSubterm().add(varr4);
			Add add6 = MultisetsFactory.eINSTANCE.createAdd();
			add6.getSubterm().add(nof35);
			add6.getSubterm().add(nof36);
			HLAnnotation hlannot27 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot27.setStructure(add6);
			hlannot27.setText("1'(l) + 1'(r)");
			ArcHLAPI join2forks = new ArcHLAPI("join2forks", new NameHLAPI("Join2Forks"), join, forks, null,
					new HLAnnotationHLAPI(hlannot27), defaultPage);
			// Forks --> Join : 1'(l) + 1'(r)
			NumberConstant one37 = IntegersFactory.eINSTANCE.createNumberConstant();
			one37.setValue(1);
			one37.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varl5 = TermsFactory.eINSTANCE.createVariable();
			varl5.setVariableDecl(lDecl.getContainedItem());
			NumberOf nof37 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof37.getSubterm().add(one37);
			nof37.getSubterm().add(varl5);
			NumberConstant one38 = IntegersFactory.eINSTANCE.createNumberConstant();
			one38.setValue(1);
			one38.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr5 = TermsFactory.eINSTANCE.createVariable();
			varr5.setVariableDecl(rDecl.getContainedItem());
			NumberOf nof38 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof38.getSubterm().add(one38);
			nof38.getSubterm().add(varr5);
			Add add7 = MultisetsFactory.eINSTANCE.createAdd();
			add7.getSubterm().add(nof37);
			add7.getSubterm().add(nof38);
			HLAnnotation hlannot28 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot28.setStructure(add7);
			hlannot28.setText("1'(l) + 1'(r)");
			ArcHLAPI forks2join = new ArcHLAPI("forks2join", new NameHLAPI("Forks2Join"), forks, join, null,
					new HLAnnotationHLAPI(hlannot28), defaultPage);
			// TakeLeft --> HasLeft : 1'(p)
			NumberConstant one39 = IntegersFactory.eINSTANCE.createNumberConstant();
			one39.setValue(1);
			one39.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp27 = TermsFactory.eINSTANCE.createVariable();
			varp27.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof39 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof39.getSubterm().add(one39);
			nof39.getSubterm().add(varp27);
			HLAnnotation hlannot29 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot29.setStructure(nof39);
			hlannot29.setText("1'(p)");
			ArcHLAPI takeleft2hasleft = new ArcHLAPI("takeleft2hasleft", new NameHLAPI("TakeLeft2HasLeft"), takeLeft, hasLeft, null,
					new HLAnnotationHLAPI(hlannot29), defaultPage);
			// HasLeft --> Eat : 1'(p)
			NumberConstant one40 = IntegersFactory.eINSTANCE.createNumberConstant();
			one40.setValue(1);
			one40.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp28 = TermsFactory.eINSTANCE.createVariable();
			varp28.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof40 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof40.getSubterm().add(one40);
			nof40.getSubterm().add(varp28);
			HLAnnotation hlannot30 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot30.setStructure(nof40);
			hlannot30.setText("1'(p)");
			ArcHLAPI hasleft2eat = new ArcHLAPI("hasleft2eat", new NameHLAPI("HasLeft2Eat"), hasLeft, eat, null,
					new HLAnnotationHLAPI(hlannot30), defaultPage);
			// HasRight --> Eat : 1'(p)
			NumberConstant one41 = IntegersFactory.eINSTANCE.createNumberConstant();
			one41.setValue(1);
			one41.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp29 = TermsFactory.eINSTANCE.createVariable();
			varp29.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof41 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof41.getSubterm().add(one41);
			nof41.getSubterm().add(varp29);
			HLAnnotation hlannot31 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot31.setStructure(nof41);
			hlannot31.setText("1'(p)");
			ArcHLAPI hasright2eat = new ArcHLAPI("hasright2eat", new NameHLAPI("HasRight2Eat"), hasRight, eat, null,
					new HLAnnotationHLAPI(hlannot31), defaultPage);
			// TakeRight --> HasRight : 1'(p)
			NumberConstant one42 = IntegersFactory.eINSTANCE.createNumberConstant();
			one42.setValue(1);
			one42.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp30 = TermsFactory.eINSTANCE.createVariable();
			varp30.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof42 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof42.getSubterm().add(one42);
			nof42.getSubterm().add(varp30);
			HLAnnotation hlannot32 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot32.setStructure(nof42);
			hlannot32.setText("1'(p)");
			ArcHLAPI takeright2hasright = new ArcHLAPI("takeright2hasright", new NameHLAPI("TakeRight2HasRight"), takeRight, hasRight, null,
					new HLAnnotationHLAPI(hlannot32), defaultPage);
			
		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}
	}
	
	
}
