package fr.lip6.mcc.pnml.generators;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.Add;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Tuple;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

@Preamble(
		author = "Lom",
		date = "24/03/2012",
		description = "Manual construction of the Petri net (ISO/IEC 15909-2) of Global Allocation Resource Management. \n",
		provider = "Fabrice Kordon"
)
public final class GlobalAllocationRes extends RootGenerator implements PnmlGenerator {

	private HLAPIRootClass gallocresDoc;

	public GlobalAllocationRes() {
		super();
	}
	
	@Override
	public void generate2Pnml(String proprietaryFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Global Allocation Resource Management instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Global Allocation Resource Management instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done : Global Allocation Resource Management instance " + instanceSize);
	}
	
	@Override
	protected void createModel(int instanceSize) {
		gallocresDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("gallocresDoc"+ instanceSize, gallocresDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("globalAllocationResourceManagement", PNTypeHLAPI.SYMNET, new NameHLAPI(
					"Global Allocation Resource Management"), (PetriNetDocHLAPI) gallocresDoc);
			PageHLAPI defaultPage = new PageHLAPI("gallocresDefaultPage", net);
			/* Net declarations */
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class Proc is 1..N (where N = instanceSize)
			CyclicEnumerationHLAPI prEnum = new CyclicEnumerationHLAPI();
			for (int i = 1; i <= instanceSize; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("prId" + String.valueOf(i), String.valueOf(i));
				prEnum.addElementsHLAPI(oneConstant);
			}
			NamedSortHLAPI prName = new NamedSortHLAPI("prclass", "Proc", prEnum, declarations);
			// Class Res is 1..M (where M = 2*instanceSize)
			CyclicEnumerationHLAPI reEnum = new CyclicEnumerationHLAPI();
			int m = 2 * instanceSize;
			for (int i = 1; i <= m ; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("reId" + String.valueOf(i), String.valueOf(i));
				reEnum.addElementsHLAPI(oneConstant);
			}
			NamedSortHLAPI reName = new NamedSortHLAPI("reclass", "Res", reEnum, declarations);
			// Domain PR is <Proc,Res>
			ProductSortHLAPI presDomain = new ProductSortHLAPI();
			presDomain.addElementSortHLAPI(new UserSortHLAPI(prName));
			presDomain.addElementSortHLAPI(new UserSortHLAPI(reName));
			NamedSortHLAPI presDomainName = new NamedSortHLAPI("pr", "PR", presDomain, declarations);
			// Var p in Proc
			VariableDeclHLAPI pDecl = new VariableDeclHLAPI("varp", "p", new UserSortHLAPI(prName), declarations);
			// Var r1, r2, r3, r4 in Res
			VariableDeclHLAPI r1Decl = new VariableDeclHLAPI("varr1", "r1", new UserSortHLAPI(reName), declarations);
			VariableDeclHLAPI r2Decl = new VariableDeclHLAPI("varr2", "r2", new UserSortHLAPI(reName), declarations);
			VariableDeclHLAPI r3Decl = new VariableDeclHLAPI("varr3", "r3", new UserSortHLAPI(reName), declarations);
			VariableDeclHLAPI r4Decl = new VariableDeclHLAPI("varr4", "r4", new UserSortHLAPI(reName), declarations);
			/* Places */
			// Place Processes {dom: Proc; mark: Proc.all}
			PlaceHLAPI processes = new PlaceHLAPI("processes", new NameHLAPI("Processes"), null, new TypeHLAPI(null, "Proc",
					new UserSortHLAPI(prName)),
					new HLMarkingHLAPI(null, "Proc.all", new AllHLAPI(new UserSortHLAPI(prName))), defaultPage);
			// Place Resources {dom: Res; mark: Res.all}
			PlaceHLAPI resources = new PlaceHLAPI("resources", new NameHLAPI("Resources"), null, new TypeHLAPI(null, "Res",
					new UserSortHLAPI(reName)),
					new HLMarkingHLAPI(null, "Res.all", new AllHLAPI(new UserSortHLAPI(reName))), defaultPage);
			// Place pr_in {dom: Proc}
			PlaceHLAPI prin = new PlaceHLAPI("prin", new NameHLAPI("pr_in"), null, new TypeHLAPI(null,
					"Proc", new UserSortHLAPI(prName)), null, defaultPage);
			// Place in_critical_s {dom: PR}
			PlaceHLAPI inCriticalS = new PlaceHLAPI("incriticals", new NameHLAPI("in_critical_s"), null, new TypeHLAPI(null,
					"PR", new UserSortHLAPI(presDomainName)), null, defaultPage);
			// Place pr_released {dom: Proc}
			PlaceHLAPI prreleased = new PlaceHLAPI("prreleased", new NameHLAPI("pr_released"), null, new TypeHLAPI(null,
					"Proc", new UserSortHLAPI(prName)), null, defaultPage);
			/* Transitions */
			// Transition enter1
			TransitionHLAPI enter1 = new TransitionHLAPI("enter1", new NameHLAPI("enter1"), null,
					null, defaultPage);
			// Transition enter2
			TransitionHLAPI enter2 = new TransitionHLAPI("enter2", new NameHLAPI("enter2"), null,
					null, defaultPage);
			// Transition enter3
			TransitionHLAPI enter3 = new TransitionHLAPI("enter3", new NameHLAPI("enter3"), null,
					null, defaultPage);
			// Transition enter4
			TransitionHLAPI enter4 = new TransitionHLAPI("enter4", new NameHLAPI("enter4"), null,
					null, defaultPage);
			// Transition exit
			TransitionHLAPI exit = new TransitionHLAPI("exit", new NameHLAPI("exit"), null,
					null, defaultPage);
			// Transition release1
			TransitionHLAPI release1 = new TransitionHLAPI("release1", new NameHLAPI("release1"), null,
					null, defaultPage);
			// Transition release2
			TransitionHLAPI release2 = new TransitionHLAPI("release2", new NameHLAPI("release2"), null,
					null, defaultPage);
			/* Arcs */
			// Processes --> enter1 : 1'(p)
			NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
			one0.setValue(1);
			one0.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp1 = TermsFactory.eINSTANCE.createVariable();
			varp1.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof0.getSubterm().add(one0);
			nof0.getSubterm().add(varp1);
			HLAnnotation hlannot0 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot0.setStructure(nof0);
			hlannot0.setText("1'(p)");
			ArcHLAPI processes2enter1 = new ArcHLAPI("processes2enter1", new NameHLAPI("Processes2enter1"), processes, enter1, null,
					new HLAnnotationHLAPI(hlannot0), defaultPage);
			// Processes --> enter2 : 1'(p)
			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp2 = TermsFactory.eINSTANCE.createVariable();
			varp2.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one2);
			nof1.getSubterm().add(varp2);
			HLAnnotation hlannot1 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot1.setStructure(nof1);
			hlannot1.setText("1'(p)");
			ArcHLAPI processes2enter2 = new ArcHLAPI("processes2enter2", new NameHLAPI("Processes2enter2"), processes, enter2, null,
					new HLAnnotationHLAPI(hlannot1), defaultPage);
			// Processes --> enter3 : 1'(p)
			NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp3 = TermsFactory.eINSTANCE.createVariable();
			varp3.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one3);
			nof2.getSubterm().add(varp3);
			HLAnnotation hlannot2 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot2.setStructure(nof2);
			hlannot2.setText("1'(p)");
			ArcHLAPI processes2enter3 = new ArcHLAPI("processes2enter3", new NameHLAPI("Processes2enter3"), processes, enter3, null,
					new HLAnnotationHLAPI(hlannot2), defaultPage);
			// Processes --> enter4 : 1'(p)
			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp4 = TermsFactory.eINSTANCE.createVariable();
			varp4.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one4);
			nof3.getSubterm().add(varp4);
			HLAnnotation hlannot3 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot3.setStructure(nof3);
			hlannot3.setText("1'(p)");
			ArcHLAPI processes2enter4 = new ArcHLAPI("processes2enter4", new NameHLAPI("Processes2enter4"), processes, enter4, null,
					new HLAnnotationHLAPI(hlannot3), defaultPage);
			// exit --> Processes : 1'(p)
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp5 = TermsFactory.eINSTANCE.createVariable();
			varp5.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one5);
			nof4.getSubterm().add(varp5);
			HLAnnotation hlannot5 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot5.setStructure(nof4);
			hlannot5.setText("1'(p)");
			ArcHLAPI exit2processes = new ArcHLAPI("exit2processes", new NameHLAPI("exit2Processes"), exit, processes, null,
					new HLAnnotationHLAPI(hlannot5), defaultPage);
			// pr_released --> exit : 4'(p)
			NumberConstant four0 = IntegersFactory.eINSTANCE.createNumberConstant();
			four0.setValue(4);
			four0.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp6 = TermsFactory.eINSTANCE.createVariable();
			varp6.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(four0);
			nof5.getSubterm().add(varp6);
			HLAnnotation hlannot6 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot6.setStructure(nof5);
			hlannot6.setText("4'(p)");
			ArcHLAPI prreleased2exit = new ArcHLAPI("prreleased2exit", new NameHLAPI("pr_released2exit"), prreleased, exit, null,
					new HLAnnotationHLAPI(hlannot6), defaultPage);
			// enter1 --> pr_released : 3'(p)
			NumberConstant three0 = IntegersFactory.eINSTANCE.createNumberConstant();
			three0.setValue(3);
			three0.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp7 = TermsFactory.eINSTANCE.createVariable();
			varp7.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(three0);
			nof6.getSubterm().add(varp7);
			HLAnnotation hlannot7 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot7.setStructure(nof6);
			hlannot7.setText("3'(p)");
			ArcHLAPI enter12prreleased = new ArcHLAPI("enter12prreleased", new NameHLAPI("enter12pr_released"), enter1, prreleased, null,
					new HLAnnotationHLAPI(hlannot7), defaultPage);
			// enter2 --> pr_released : 2'(p)
			NumberConstant two0 = IntegersFactory.eINSTANCE.createNumberConstant();
			two0.setValue(2);
			two0.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp8 = TermsFactory.eINSTANCE.createVariable();
			varp8.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(two0);
			nof7.getSubterm().add(varp8);
			HLAnnotation hlannot8 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot8.setStructure(nof7);
			hlannot8.setText("2'(p)");
			ArcHLAPI enter22prreleased = new ArcHLAPI("enter22prreleased", new NameHLAPI("enter22pr_released"), enter2, prreleased, null,
					new HLAnnotationHLAPI(hlannot8), defaultPage);
			// enter3 --> pr_released : 1'(p)
			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp9 = TermsFactory.eINSTANCE.createVariable();
			varp9.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one6);
			nof8.getSubterm().add(varp9);
			HLAnnotation hlannot9 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot9.setStructure(nof8);
			hlannot9.setText("1'(p)");
			ArcHLAPI enter32prreleased = new ArcHLAPI("enter32prreleased", new NameHLAPI("enter32pr_released"), enter3, prreleased, null,
					new HLAnnotationHLAPI(hlannot9), defaultPage);
			// release1--> pr_released : 1'(p)
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp10 = TermsFactory.eINSTANCE.createVariable();
			varp10.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof9 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof9.getSubterm().add(one7);
			nof9.getSubterm().add(varp10);
			HLAnnotation hlannot10 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot10.setStructure(nof9);
			hlannot10.setText("1'(p)");
			ArcHLAPI release12prreleased = new ArcHLAPI("release12prreleased", new NameHLAPI("release12pr_released"), release1, prreleased, null,
					new HLAnnotationHLAPI(hlannot10), defaultPage);
			// release2--> pr_released : 2'(p)
			NumberConstant two1 = IntegersFactory.eINSTANCE.createNumberConstant();
			two1.setValue(2);
			two1.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp11 = TermsFactory.eINSTANCE.createVariable();
			varp11.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof10 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof10.getSubterm().add(two1);
			nof10.getSubterm().add(varp11);
			HLAnnotation hlannot11 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot11.setStructure(nof10);
			hlannot11.setText("2'(p)");
			ArcHLAPI release22prreleased = new ArcHLAPI("release22prreleased", new NameHLAPI("release22pr_released"), release2, prreleased, null,
					new HLAnnotationHLAPI(hlannot11), defaultPage);
			// release1 --> Resources : 1'(r1)
			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr10 = TermsFactory.eINSTANCE.createVariable();
			varr10.setVariableDecl(r1Decl.getContainedItem());
			NumberOf nof11 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof11.getSubterm().add(one8);
			nof11.getSubterm().add(varr10);
			HLAnnotation hlannot12 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot12.setStructure(nof11);
			hlannot12.setText("1'(r1)");
			ArcHLAPI release12resources = new ArcHLAPI("release12resources", new NameHLAPI("release12Resources"), release1, resources, null,
					new HLAnnotationHLAPI(hlannot12), defaultPage);
			// release2 --> Resources : 1'(r1) + 1'(r2)
			NumberConstant one9 = IntegersFactory.eINSTANCE.createNumberConstant();
			one9.setValue(1);
			one9.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr11 = TermsFactory.eINSTANCE.createVariable();
			varr11.setVariableDecl(r1Decl.getContainedItem());
			NumberOf nof12 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof12.getSubterm().add(one9);
			nof12.getSubterm().add(varr11);

			NumberConstant one10 = IntegersFactory.eINSTANCE.createNumberConstant();
			one10.setValue(1);
			one10.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr20 = TermsFactory.eINSTANCE.createVariable();
			varr20.setVariableDecl(r2Decl.getContainedItem());
			NumberOf nof13 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof13.getSubterm().add(one10);
			nof13.getSubterm().add(varr20);

			Add add0 = MultisetsFactory.eINSTANCE.createAdd();
			add0.getSubterm().add(nof12);
			add0.getSubterm().add(nof13);

			HLAnnotation hlannot13 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot13.setStructure(add0);
			hlannot13.setText("1'(r1) + 1'(r2)");
			ArcHLAPI release22Resources = new ArcHLAPI("release22Resources", new NameHLAPI("release22Resources"),
					release2, resources, null, new HLAnnotationHLAPI(hlannot13), defaultPage);
			// pr_in --> release1 : 1'(p)
			NumberConstant one11 = IntegersFactory.eINSTANCE.createNumberConstant();
			one11.setValue(1);
			one11.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp12 = TermsFactory.eINSTANCE.createVariable();
			varp12.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof14 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof14.getSubterm().add(one11);
			nof14.getSubterm().add(varp12);
			HLAnnotation hlannot14 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot14.setStructure(nof14);
			hlannot14.setText("1'(p)");
			ArcHLAPI prin2release1 = new ArcHLAPI("prin2release1", new NameHLAPI("pr_in2release1"), prin, release1, null,
					new HLAnnotationHLAPI(hlannot14), defaultPage);
			// pr_in --> release2 : 2'(p)
			NumberConstant two2 = IntegersFactory.eINSTANCE.createNumberConstant();
			two2.setValue(2);
			two2.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp13 = TermsFactory.eINSTANCE.createVariable();
			varp13.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof15 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof15.getSubterm().add(two2);
			nof15.getSubterm().add(varp13);
			HLAnnotation hlannot15 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot15.setStructure(nof15);
			hlannot15.setText("2'(p)");
			ArcHLAPI prin2release2 = new ArcHLAPI("prin2release2", new NameHLAPI("pr_in2release2"), prin, release2, null,
					new HLAnnotationHLAPI(hlannot15), defaultPage);
			// in_critical_s --> release1 : 1'(p,r1)
			NumberConstant one12 = IntegersFactory.eINSTANCE.createNumberConstant();
			one12.setValue(1);
			one12.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp14 = TermsFactory.eINSTANCE.createVariable();
			varp14.setVariableDecl(pDecl.getContainedItem());
			Variable varr12 = TermsFactory.eINSTANCE.createVariable();
			varr12.setVariableDecl(r1Decl.getContainedItem());
			Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
			tuple0.getSubterm().add(varp14);
			tuple0.getSubterm().add(varr12);
			NumberOf nof16 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof16.getSubterm().add(one12);
			nof16.getSubterm().add(tuple0);
			HLAnnotation hlannot16 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot16.setStructure(nof16);
			hlannot16.setText("1'(p,r1)");
			ArcHLAPI incriticals2release1 = new ArcHLAPI("incriticals2release1", new NameHLAPI("in_critical_s2release1"), inCriticalS, release1,
					null, new HLAnnotationHLAPI(hlannot16), defaultPage);
			// in_critical_s --> release2 : 1'(p,r1) + 1'(p,r2)
			NumberConstant one13 = IntegersFactory.eINSTANCE.createNumberConstant();
			one13.setValue(1);
			one13.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp15 = TermsFactory.eINSTANCE.createVariable();
			varp15.setVariableDecl(pDecl.getContainedItem());
			Variable varr13 = TermsFactory.eINSTANCE.createVariable();
			varr13.setVariableDecl(r1Decl.getContainedItem());
			Tuple tuple1 = TermsFactory.eINSTANCE.createTuple();
			tuple1.getSubterm().add(varp15);
			tuple1.getSubterm().add(varr13);
			NumberOf nof17 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof17.getSubterm().add(one13);
			nof17.getSubterm().add(tuple1);

			NumberConstant one14 = IntegersFactory.eINSTANCE.createNumberConstant();
			one14.setValue(1);
			one14.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp16 = TermsFactory.eINSTANCE.createVariable();
			varp16.setVariableDecl(pDecl.getContainedItem());
			Variable varr21 = TermsFactory.eINSTANCE.createVariable();
			varr21.setVariableDecl(r2Decl.getContainedItem());
			Tuple tuple2 = TermsFactory.eINSTANCE.createTuple();
			tuple2.getSubterm().add(varp16);
			tuple2.getSubterm().add(varr21);
			NumberOf nof18 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof18.getSubterm().add(one14);
			nof18.getSubterm().add(tuple2);

			Add add1 = MultisetsFactory.eINSTANCE.createAdd();
			add1.getSubterm().add(nof17);
			add1.getSubterm().add(nof18);

			HLAnnotation hlannot17 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot17.setStructure(add1);
			hlannot17.setText("1'(p,r1) + 1'(p,r2)");
			ArcHLAPI incriticals2release2 = new ArcHLAPI("incriticals2release2", new NameHLAPI("in_critical_s2release2"), inCriticalS,
					release2, null, new HLAnnotationHLAPI(hlannot17), defaultPage);
			// enter1 --> pr_in : 1'(p) 
			NumberConstant one15 = IntegersFactory.eINSTANCE.createNumberConstant();
			one15.setValue(1);
			one15.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp17 = TermsFactory.eINSTANCE.createVariable();
			varp17.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof19 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof19.getSubterm().add(one15);
			nof19.getSubterm().add(varp17);
			HLAnnotation hlannot18 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot18.setStructure(nof19);
			hlannot18.setText("1'(p)");
			ArcHLAPI enter12prin = new ArcHLAPI("enter12prin", new NameHLAPI("enter12pr_in"), enter1, prin, null,
					new HLAnnotationHLAPI(hlannot18), defaultPage);
			// enter2 --> pr_in : 2'(p)
			NumberConstant two3 = IntegersFactory.eINSTANCE.createNumberConstant();
			two3.setValue(2);
			two3.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp18 = TermsFactory.eINSTANCE.createVariable();
			varp18.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof20 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof20.getSubterm().add(two3);
			nof20.getSubterm().add(varp18);
			HLAnnotation hlannot19 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot19.setStructure(nof20);
			hlannot19.setText("2'(p)");
			ArcHLAPI enter22prin = new ArcHLAPI("enter22prin", new NameHLAPI("enter22pr_in"), enter2, prin, null,
					new HLAnnotationHLAPI(hlannot19), defaultPage);
			// enter3 --> pr_in : 3'(p)
			NumberConstant three1 = IntegersFactory.eINSTANCE.createNumberConstant();
			three1.setValue(3);
			three1.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp19 = TermsFactory.eINSTANCE.createVariable();
			varp19.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof21 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof21.getSubterm().add(three1);
			nof21.getSubterm().add(varp19);
			HLAnnotation hlannot20 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot20.setStructure(nof21);
			hlannot20.setText("3'(p)");
			ArcHLAPI enter32prin = new ArcHLAPI("enter32prin", new NameHLAPI("enter32pr_in"), enter3, prin, null,
					new HLAnnotationHLAPI(hlannot20), defaultPage);
			// enter4 --> pr_in : 4'(p)
			NumberConstant four1 = IntegersFactory.eINSTANCE.createNumberConstant();
			four1.setValue(4);
			four1.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp20 = TermsFactory.eINSTANCE.createVariable();
			varp20.setVariableDecl(pDecl.getContainedItem());
			NumberOf nof22 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof22.getSubterm().add(four1);
			nof22.getSubterm().add(varp20);
			HLAnnotation hlannot21 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot21.setStructure(nof22);
			hlannot21.setText("4'(p)");
			ArcHLAPI enter42prin = new ArcHLAPI("enter42prin", new NameHLAPI("enter42pr_in"), enter4, prin, null,
					new HLAnnotationHLAPI(hlannot21), defaultPage);
			// enter4 --> in_critical_s : 1'(p,r1) + 1'(p,r2) + 1'(p,r3) + 1'(p,r4)
			NumberConstant one16 = IntegersFactory.eINSTANCE.createNumberConstant();
			one16.setValue(1);
			one16.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp21 = TermsFactory.eINSTANCE.createVariable();
			varp21.setVariableDecl(pDecl.getContainedItem());
			Variable varr14 = TermsFactory.eINSTANCE.createVariable();
			varr14.setVariableDecl(r1Decl.getContainedItem());
			Tuple tuple3 = TermsFactory.eINSTANCE.createTuple();
			tuple3.getSubterm().add(varp21);
			tuple3.getSubterm().add(varr14);
			NumberOf nof23 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof23.getSubterm().add(one16);
			nof23.getSubterm().add(tuple3);

			NumberConstant one17 = IntegersFactory.eINSTANCE.createNumberConstant();
			one17.setValue(1);
			one17.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp22 = TermsFactory.eINSTANCE.createVariable();
			varp22.setVariableDecl(pDecl.getContainedItem());
			Variable varr22 = TermsFactory.eINSTANCE.createVariable();
			varr22.setVariableDecl(r2Decl.getContainedItem());
			Tuple tuple4 = TermsFactory.eINSTANCE.createTuple();
			tuple4.getSubterm().add(varp22);
			tuple4.getSubterm().add(varr22);
			NumberOf nof24 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof24.getSubterm().add(one17);
			nof24.getSubterm().add(tuple4);
			
			NumberConstant one18 = IntegersFactory.eINSTANCE.createNumberConstant();
			one18.setValue(1);
			one18.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp23 = TermsFactory.eINSTANCE.createVariable();
			varp23.setVariableDecl(pDecl.getContainedItem());
			Variable varr31 = TermsFactory.eINSTANCE.createVariable();
			varr31.setVariableDecl(r3Decl.getContainedItem());
			Tuple tuple5 = TermsFactory.eINSTANCE.createTuple();
			tuple5.getSubterm().add(varp23);
			tuple5.getSubterm().add(varr31);
			NumberOf nof25 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof25.getSubterm().add(one18);
			nof25.getSubterm().add(tuple5);
			
			NumberConstant one19 = IntegersFactory.eINSTANCE.createNumberConstant();
			one19.setValue(1);
			one19.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp24 = TermsFactory.eINSTANCE.createVariable();
			varp24.setVariableDecl(pDecl.getContainedItem());
			Variable varr40 = TermsFactory.eINSTANCE.createVariable();
			varr40.setVariableDecl(r4Decl.getContainedItem());
			Tuple tuple6 = TermsFactory.eINSTANCE.createTuple();
			tuple6.getSubterm().add(varp24);
			tuple6.getSubterm().add(varr40);
			NumberOf nof26 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof26.getSubterm().add(one19);
			nof26.getSubterm().add(tuple6);


			Add add2 = MultisetsFactory.eINSTANCE.createAdd();
			add2.getSubterm().add(nof23);
			add2.getSubterm().add(nof24);
			add2.getSubterm().add(nof25);
			add2.getSubterm().add(nof26);

			HLAnnotation hlannot22 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot22.setStructure(add2);
			hlannot22.setText("1'(p,r1) + 1'(p,r2) + 1'(p,r3) + 1'(p,r4)");
			ArcHLAPI enter42incriticals = new ArcHLAPI("enter42incriticals", new NameHLAPI("enter42in_critical_s"), enter4, inCriticalS,
					 null, new HLAnnotationHLAPI(hlannot22), defaultPage);
			// enter3 --> in_critical_s : 1'(p,r1) + 1'(p,r2) + 1'(p,r3)
			NumberConstant one20 = IntegersFactory.eINSTANCE.createNumberConstant();
			one20.setValue(1);
			one20.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp25 = TermsFactory.eINSTANCE.createVariable();
			varp25.setVariableDecl(pDecl.getContainedItem());
			Variable varr15 = TermsFactory.eINSTANCE.createVariable();
			varr15.setVariableDecl(r1Decl.getContainedItem());
			Tuple tuple7 = TermsFactory.eINSTANCE.createTuple();
			tuple7.getSubterm().add(varp25);
			tuple7.getSubterm().add(varr15);
			NumberOf nof27 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof27.getSubterm().add(one20);
			nof27.getSubterm().add(tuple7);

			NumberConstant one21 = IntegersFactory.eINSTANCE.createNumberConstant();
			one21.setValue(1);
			one21.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp26 = TermsFactory.eINSTANCE.createVariable();
			varp26.setVariableDecl(pDecl.getContainedItem());
			Variable varr23 = TermsFactory.eINSTANCE.createVariable();
			varr23.setVariableDecl(r2Decl.getContainedItem());
			Tuple tuple8 = TermsFactory.eINSTANCE.createTuple();
			tuple8.getSubterm().add(varp26);
			tuple8.getSubterm().add(varr23);
			NumberOf nof28 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof28.getSubterm().add(one21);
			nof28.getSubterm().add(tuple8);
			
			NumberConstant one22 = IntegersFactory.eINSTANCE.createNumberConstant();
			one22.setValue(1);
			one22.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp27 = TermsFactory.eINSTANCE.createVariable();
			varp27.setVariableDecl(pDecl.getContainedItem());
			Variable varr32 = TermsFactory.eINSTANCE.createVariable();
			varr32.setVariableDecl(r3Decl.getContainedItem());
			Tuple tuple9 = TermsFactory.eINSTANCE.createTuple();
			tuple9.getSubterm().add(varp27);
			tuple9.getSubterm().add(varr32);
			NumberOf nof29 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof29.getSubterm().add(one22);
			nof29.getSubterm().add(tuple9);
			
			Add add3 = MultisetsFactory.eINSTANCE.createAdd();
			add3.getSubterm().add(nof27);
			add3.getSubterm().add(nof28);
			add3.getSubterm().add(nof29);

			HLAnnotation hlannot23 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot23.setStructure(add3);
			hlannot23.setText("1'(p,r1) + 1'(p,r2) + 1'(p,r3)");
			ArcHLAPI enter32incriticals = new ArcHLAPI("enter32incriticals", new NameHLAPI("enter32in_critical_s"), enter3, inCriticalS,
					 null, new HLAnnotationHLAPI(hlannot23), defaultPage);
			// enter2 --> in_critical_s : 1'(p,r1) + 1'(p,r2)
			NumberConstant one23 = IntegersFactory.eINSTANCE.createNumberConstant();
			one23.setValue(1);
			one23.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp28 = TermsFactory.eINSTANCE.createVariable();
			varp28.setVariableDecl(pDecl.getContainedItem());
			Variable varr16 = TermsFactory.eINSTANCE.createVariable();
			varr16.setVariableDecl(r1Decl.getContainedItem());
			Tuple tuple10 = TermsFactory.eINSTANCE.createTuple();
			tuple10.getSubterm().add(varp28);
			tuple10.getSubterm().add(varr16);
			NumberOf nof30 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof30.getSubterm().add(one23);
			nof30.getSubterm().add(tuple10);

			NumberConstant one24 = IntegersFactory.eINSTANCE.createNumberConstant();
			one24.setValue(1);
			one24.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp29 = TermsFactory.eINSTANCE.createVariable();
			varp29.setVariableDecl(pDecl.getContainedItem());
			Variable varr24 = TermsFactory.eINSTANCE.createVariable();
			varr24.setVariableDecl(r2Decl.getContainedItem());
			Tuple tuple11 = TermsFactory.eINSTANCE.createTuple();
			tuple11.getSubterm().add(varp29);
			tuple11.getSubterm().add(varr24);
			NumberOf nof31 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof31.getSubterm().add(one24);
			nof31.getSubterm().add(tuple11);
			
			Add add4 = MultisetsFactory.eINSTANCE.createAdd();
			add4.getSubterm().add(nof30);
			add4.getSubterm().add(nof31);

			HLAnnotation hlannot24 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot24.setStructure(add4);
			hlannot24.setText("1'(p,r1) + 1'(p,r2)");
			ArcHLAPI enter22incriticals = new ArcHLAPI("enter22incriticals", new NameHLAPI("enter22in_critical_s"), enter2, inCriticalS,
					 null, new HLAnnotationHLAPI(hlannot24), defaultPage);
			// enter1 --> in_critical_s : 1'(p,r1)
			NumberConstant one25 = IntegersFactory.eINSTANCE.createNumberConstant();
			one25.setValue(1);
			one25.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varp30 = TermsFactory.eINSTANCE.createVariable();
			varp30.setVariableDecl(pDecl.getContainedItem());
			Variable varr17 = TermsFactory.eINSTANCE.createVariable();
			varr17.setVariableDecl(r1Decl.getContainedItem());
			Tuple tuple12 = TermsFactory.eINSTANCE.createTuple();
			tuple12.getSubterm().add(varp30);
			tuple12.getSubterm().add(varr17);
			NumberOf nof32 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof32.getSubterm().add(one25);
			nof32.getSubterm().add(tuple12);
			HLAnnotation hlannot25 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot25.setStructure(nof32);
			hlannot25.setText("1'(p,r1)");
			ArcHLAPI enter12incriticals = new ArcHLAPI("enter12incriticals", new NameHLAPI("enter12in_critical_s"), enter1, inCriticalS,
					 null, new HLAnnotationHLAPI(hlannot25), defaultPage);
			// resources --> enter4 : 1'(r1) + 1'(r2) + 1'(r3) + 1'(r4) 
			NumberConstant one26 = IntegersFactory.eINSTANCE.createNumberConstant();
			one26.setValue(1);
			one26.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr18 = TermsFactory.eINSTANCE.createVariable();
			varr18.setVariableDecl(r1Decl.getContainedItem());
			NumberOf nof33 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof33.getSubterm().add(one26);
			nof33.getSubterm().add(varr18);

			NumberConstant one27 = IntegersFactory.eINSTANCE.createNumberConstant();
			one27.setValue(1);
			one27.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr25 = TermsFactory.eINSTANCE.createVariable();
			varr25.setVariableDecl(r2Decl.getContainedItem());
			NumberOf nof34 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof34.getSubterm().add(one27);
			nof34.getSubterm().add(varr25);
			
			NumberConstant one28 = IntegersFactory.eINSTANCE.createNumberConstant();
			one28.setValue(1);
			one28.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr33 = TermsFactory.eINSTANCE.createVariable();
			varr33.setVariableDecl(r3Decl.getContainedItem());
			NumberOf nof35 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof35.getSubterm().add(one28);
			nof35.getSubterm().add(varr33);
			
			NumberConstant one29 = IntegersFactory.eINSTANCE.createNumberConstant();
			one29.setValue(1);
			one29.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr41 = TermsFactory.eINSTANCE.createVariable();
			varr41.setVariableDecl(r4Decl.getContainedItem());
			NumberOf nof36 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof36.getSubterm().add(one29);
			nof36.getSubterm().add(varr41);

			Add add5 = MultisetsFactory.eINSTANCE.createAdd();
			add5.getSubterm().add(nof33);
			add5.getSubterm().add(nof34);
			add5.getSubterm().add(nof35);
			add5.getSubterm().add(nof36);

			HLAnnotation hlannot26 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot26.setStructure(add5);
			hlannot26.setText("1'(r1) + 1'(r2) + 1'(r3) + 1'(r4)");
			ArcHLAPI resources2enter4 = new ArcHLAPI("resources2enter4", new NameHLAPI("Resources2enter4"),
				 resources, enter4, null, new HLAnnotationHLAPI(hlannot26), defaultPage);
			// resources --> enter3 : 1'(r1) + 1'(r2) + 1'(r3) 
			NumberConstant one30 = IntegersFactory.eINSTANCE.createNumberConstant();
			one30.setValue(1);
			one30.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr19 = TermsFactory.eINSTANCE.createVariable();
			varr19.setVariableDecl(r1Decl.getContainedItem());
			NumberOf nof37 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof37.getSubterm().add(one30);
			nof37.getSubterm().add(varr19);

			NumberConstant one31 = IntegersFactory.eINSTANCE.createNumberConstant();
			one31.setValue(1);
			one31.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr26 = TermsFactory.eINSTANCE.createVariable();
			varr26.setVariableDecl(r2Decl.getContainedItem());
			NumberOf nof38 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof38.getSubterm().add(one31);
			nof38.getSubterm().add(varr26);
			
			NumberConstant one32 = IntegersFactory.eINSTANCE.createNumberConstant();
			one32.setValue(1);
			one32.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr34 = TermsFactory.eINSTANCE.createVariable();
			varr34.setVariableDecl(r3Decl.getContainedItem());
			NumberOf nof39 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof39.getSubterm().add(one32);
			nof39.getSubterm().add(varr34);

			Add add6 = MultisetsFactory.eINSTANCE.createAdd();
			add6.getSubterm().add(nof37);
			add6.getSubterm().add(nof38);
			add6.getSubterm().add(nof39);

			HLAnnotation hlannot27 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot27.setStructure(add6);
			hlannot27.setText("1'(r1) + 1'(r2) + 1'(r3)");
			ArcHLAPI resources2enter3 = new ArcHLAPI("resources2enter3", new NameHLAPI("Resources2enter3"),
				 resources, enter3, null, new HLAnnotationHLAPI(hlannot27), defaultPage);
			// resources --> enter2 : 1'(r1) + 1'(r2)
			NumberConstant one33 = IntegersFactory.eINSTANCE.createNumberConstant();
			one33.setValue(1);
			one33.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr110 = TermsFactory.eINSTANCE.createVariable();
			varr110.setVariableDecl(r1Decl.getContainedItem());
			NumberOf nof40 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof40.getSubterm().add(one33);
			nof40.getSubterm().add(varr110);

			NumberConstant one34 = IntegersFactory.eINSTANCE.createNumberConstant();
			one34.setValue(1);
			one34.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr27 = TermsFactory.eINSTANCE.createVariable();
			varr27.setVariableDecl(r2Decl.getContainedItem());
			NumberOf nof41 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof41.getSubterm().add(one34);
			nof41.getSubterm().add(varr27);

			Add add7 = MultisetsFactory.eINSTANCE.createAdd();
			add7.getSubterm().add(nof40);
			add7.getSubterm().add(nof41);

			HLAnnotation hlannot28 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot28.setStructure(add7);
			hlannot28.setText("1'(r1) + 1'(r2)");
			ArcHLAPI resources2enter2 = new ArcHLAPI("resources2enter2", new NameHLAPI("Resources2enter2"),
				 resources, enter2, null, new HLAnnotationHLAPI(hlannot28), defaultPage);
			// resources --> enter2 : 1'(r1)
			NumberConstant one35 = IntegersFactory.eINSTANCE.createNumberConstant();
			one35.setValue(1);
			one35.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varr111 = TermsFactory.eINSTANCE.createVariable();
			varr111.setVariableDecl(r1Decl.getContainedItem());
			NumberOf nof42 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof42.getSubterm().add(one35);
			nof42.getSubterm().add(varr111);
			HLAnnotation hlannot29 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot29.setStructure(nof42);
			hlannot29.setText("1'(r1)");
			ArcHLAPI resources2enter1 = new ArcHLAPI("resources2enter1", new NameHLAPI("Resources2enter1"),
				 resources, enter1, null, new HLAnnotationHLAPI(hlannot29), defaultPage);
			
		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}

	}
}
