package fr.lip6.mcc.pnml.generators;

import fr.lip6.mcc.annotations.Preamble;
import fr.lip6.mcc.pnml.exceptions.MCCException;
import fr.lip6.mcc.pnml.generator.intf.PnmlGenerator;
import fr.lip6.move.pnml.framework.hlapi.HLAPIRootClass;
import fr.lip6.move.pnml.framework.utils.exception.InvalidIDException;
import fr.lip6.move.pnml.framework.utils.exception.VoidRepositoryException;
import fr.lip6.move.pnml.symmetricnet.booleans.BooleansFactory;
import fr.lip6.move.pnml.symmetricnet.booleans.Equality;
import fr.lip6.move.pnml.symmetricnet.booleans.Inequality;
import fr.lip6.move.pnml.symmetricnet.cyclicEnumerations.hlapi.CyclicEnumerationHLAPI;
import fr.lip6.move.pnml.symmetricnet.dots.DotConstant;
import fr.lip6.move.pnml.symmetricnet.dots.DotsFactory;
import fr.lip6.move.pnml.symmetricnet.dots.hlapi.DotHLAPI;
import fr.lip6.move.pnml.symmetricnet.finiteEnumerations.hlapi.FEConstantHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.Condition;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HLAnnotation;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.HlcorestructureFactory;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ArcHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.ConditionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.DeclarationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLAnnotationHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.HLMarkingHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.NameHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PNTypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PageHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PlaceHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TransitionHLAPI;
import fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.TypeHLAPI;
import fr.lip6.move.pnml.symmetricnet.integers.IntegersFactory;
import fr.lip6.move.pnml.symmetricnet.integers.NumberConstant;
import fr.lip6.move.pnml.symmetricnet.multisets.MultisetsFactory;
import fr.lip6.move.pnml.symmetricnet.multisets.NumberOf;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.AllHLAPI;
import fr.lip6.move.pnml.symmetricnet.multisets.hlapi.NumberOfHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.TermsFactory;
import fr.lip6.move.pnml.symmetricnet.terms.Tuple;
import fr.lip6.move.pnml.symmetricnet.terms.Variable;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.DeclarationsHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.NamedSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.ProductSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.UserSortHLAPI;
import fr.lip6.move.pnml.symmetricnet.terms.hlapi.VariableDeclHLAPI;

@Preamble(
		author = "Lom",
		date = "21/02/2011",
		description = "Manual construction of the Petri net (ISO/IEC 15909-2) of the Shared Memory model. \n",
		provider = "Denis Poitrenaud and Fabrice Kordon."
)
public class SharedMemory extends RootGenerator implements PnmlGenerator {
	/**
	 * Parameter value for domain size.
	 */
	protected static final int DOMAIN_SIZE = 2;

	/**
	 * Root doc Class for the net
	 */
	protected HLAPIRootClass memDoc;

	public SharedMemory() {
		super();
	}

	@Override
	public void generate2Pnml(String proprietaryFile, String pnmlFile, int instanceSize) {
		JOURNAL.info("Creating Shared Memory instance " + instanceSize + "...");
		createModel(instanceSize);
		try {
			JOURNAL.info("Exporting Shared Memory instance " + instanceSize + "...");
			exportToPNML(pnmlFile);
		} catch (MCCException e) {
			e.printStackTrace();
		}
		JOURNAL.info("Done: Shared Memory instance " + instanceSize);
	}
	
	protected void createModel(int instanceSize) {
		memDoc = new fr.lip6.move.pnml.symmetricnet.hlcorestructure.hlapi.PetriNetDocHLAPI();
		try {
			MR.createDocumentWorkspace("memoryDoc", memDoc);
			PetriNetHLAPI net = new PetriNetHLAPI("sharedMemory", PNTypeHLAPI.SYMNET,
					new NameHLAPI("Shared Memory"), (PetriNetDocHLAPI) memDoc);
			PageHLAPI defaultPage = new PageHLAPI("memDefaultPage", net);

			/* Net declarations */
			DeclarationHLAPI netDeclaration = new DeclarationHLAPI(net);
			DeclarationsHLAPI declarations = new DeclarationsHLAPI(netDeclaration);
			// Class P is 1..N
			CyclicEnumerationHLAPI pEnum = new CyclicEnumerationHLAPI();
			for (int i = 1; i <= instanceSize; i++) {
				FEConstantHLAPI oneConstant = new FEConstantHLAPI("pId" + String.valueOf(i), String.valueOf(i));
				pEnum.addElementsHLAPI(oneConstant);
			}

			NamedSortHLAPI pName = new NamedSortHLAPI("pclass", "P", pEnum, declarations);
			// Dot declaration for non-coloured annotations
			DotHLAPI dot = new DotHLAPI();
			NamedSortHLAPI dotName = new NamedSortHLAPI("dot", "Dot", dot, declarations);
			// Domain PxP is <P,P>
			ProductSortHLAPI pDomain = new ProductSortHLAPI();
			for (int i = 0; i < DOMAIN_SIZE; i++) {
				pDomain.addElementSortHLAPI(new UserSortHLAPI(pName));
			}
			NamedSortHLAPI pDomainName = new NamedSortHLAPI("pDomainId", "PxP", pDomain, declarations);
			// Var x in P
			VariableDeclHLAPI xDecl = new VariableDeclHLAPI("varx", "x", new UserSortHLAPI(pName), declarations);
			// Var y in P
			VariableDeclHLAPI yDecl = new VariableDeclHLAPI("vary", "y", new UserSortHLAPI(pName), declarations);
			// Var m in P
			VariableDeclHLAPI mDecl = new VariableDeclHLAPI("varm", "m", new UserSortHLAPI(pName), declarations);

			/* Places */
			// Place OwnMemAcc; {dom: P}
			PlaceHLAPI ownMemAcc = new PlaceHLAPI("ownMemAcc", new NameHLAPI("OwnMemAcc"), null, new TypeHLAPI(null,
					"P", new UserSortHLAPI(pName)), null, defaultPage);
			// Place Queue; {dom: P}
			PlaceHLAPI queue = new PlaceHLAPI("queue", new NameHLAPI("Queue"), null, new TypeHLAPI(null, "P",
					new UserSortHLAPI(pName)), null, defaultPage);
			// Place Memory; {dom: P; mark: P.all}
			PlaceHLAPI memory = new PlaceHLAPI("memory", new NameHLAPI("Memory"), null, new TypeHLAPI(null, "P",
					new UserSortHLAPI(pName)),
					new HLMarkingHLAPI(null, "P.all", new AllHLAPI(new UserSortHLAPI(pName))), defaultPage);
			// Place Active; {dom: P; mark: P.all}
			PlaceHLAPI active = new PlaceHLAPI("active", new NameHLAPI("Active"), null, new TypeHLAPI(null, "P",
					new UserSortHLAPI(pName)),
					new HLMarkingHLAPI(null, "P.all", new AllHLAPI(new UserSortHLAPI(pName))), defaultPage);
			// Place Ext_Mem_Acc; {dom: PxP}
			PlaceHLAPI extMemAcc = new PlaceHLAPI("extMemAcc", new NameHLAPI("Ext_Mem_Acc"), null, new TypeHLAPI(null,
					"PxP", new UserSortHLAPI(pDomainName)), null, defaultPage);
			// Place Ext_Bus; {dom: dot; mark: 1'(dot)}
			NumberConstant one0 = IntegersFactory.eINSTANCE.createNumberConstant();
			one0.setValue(1);
			one0.setType(IntegersFactory.eINSTANCE.createPositive());
			DotConstant dotConstant = DotsFactory.eINSTANCE.createDotConstant();
			NumberOf nof0 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof0.getSubterm().add(one0);
			nof0.getSubterm().add(dotConstant);
			PlaceHLAPI extBus = new PlaceHLAPI("extBus", new NameHLAPI("Ext_Bus"), null, new TypeHLAPI(null, "Dot",
					new UserSortHLAPI(dotName)), new HLMarkingHLAPI(null, "1'(dot)", new NumberOfHLAPI(nof0)),
					defaultPage);

			/* Tansitions */
			// Transition Begin_Own_Acc
			TransitionHLAPI beginOwnAcc = new TransitionHLAPI("beginOwnAcc", new NameHLAPI("Begin_Own_Acc"), null,
					null, defaultPage);
			// Transition Req_Ext_Acc
			TransitionHLAPI reqExtAcc = new TransitionHLAPI("reqExtAcc", new NameHLAPI("Req_Ext_Acc"), null, null,
					defaultPage);
			// Transition End_Ext_Acc
			TransitionHLAPI endExtAcc = new TransitionHLAPI("endExtAcc", new NameHLAPI("End_Ext_Acc"), null, null,
					defaultPage);
			// Transition End_Own_Acc {guard: m = x}
			Variable varm0 = TermsFactory.eINSTANCE.createVariable();
			varm0.setVariableDecl(mDecl.getContainedItem());
			Variable varx0 = TermsFactory.eINSTANCE.createVariable();
			varx0.setVariableDecl(xDecl.getContainedItem());
			Equality eq0 = BooleansFactory.eINSTANCE.createEquality();
			eq0.getSubterm().add(varm0);
			eq0.getSubterm().add(varx0);
			Condition endOwnAccCond = HlcorestructureFactory.eINSTANCE.createCondition();
			endOwnAccCond.setStructure(eq0);
			endOwnAccCond.setText("[m eq x]");
			ConditionHLAPI endOwnAccCondition = new ConditionHLAPI(endOwnAccCond);
			TransitionHLAPI endOwnAcc = new TransitionHLAPI("endOwnAcc", new NameHLAPI("End_Own_Acc"), null,
					endOwnAccCondition, defaultPage);
			// Transition Begin_Ext_Acc {guard: x <> m}
			Variable varm1 = TermsFactory.eINSTANCE.createVariable();
			varm1.setVariableDecl(mDecl.getContainedItem());
			Variable varx1 = TermsFactory.eINSTANCE.createVariable();
			varx1.setVariableDecl(xDecl.getContainedItem());
			Inequality ineq0 = BooleansFactory.eINSTANCE.createInequality();
			ineq0.getSubterm().add(varx1);
			ineq0.getSubterm().add(varm1);
			Condition beginExtAccCond = HlcorestructureFactory.eINSTANCE.createCondition();
			beginExtAccCond.setStructure(ineq0);
			beginExtAccCond.setText("[x ineq m]");
			ConditionHLAPI beginExtAccCondition = new ConditionHLAPI(beginExtAccCond);
			TransitionHLAPI beginExtAcc = new TransitionHLAPI("beginExtAcc", new NameHLAPI("Begin_Ext_Acc"), null,
					beginExtAccCondition, defaultPage);

			/* Arcs */
			// OwnMemAcc -> End_Own_Acc : 1'(x)
			NumberConstant one1 = IntegersFactory.eINSTANCE.createNumberConstant();
			one1.setValue(1);
			one1.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx2 = TermsFactory.eINSTANCE.createVariable();
			varx2.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof1 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof1.getSubterm().add(one1);
			nof1.getSubterm().add(varx2);
			HLAnnotation hlannot0 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot0.setStructure(nof1);
			hlannot0.setText("1'(x)");
			ArcHLAPI oma2eoa = new ArcHLAPI("oma2eoa", new NameHLAPI("OwnMemAcc2End_Own_Acc"), ownMemAcc, endOwnAcc,
					null, new HLAnnotationHLAPI(hlannot0), defaultPage);
			// Memory --> End_Own_Acc : 1'(m)
			NumberConstant one2 = IntegersFactory.eINSTANCE.createNumberConstant();
			one2.setValue(1);
			one2.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varm2 = TermsFactory.eINSTANCE.createVariable();
			varm2.setVariableDecl(mDecl.getContainedItem());
			NumberOf nof2 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof2.getSubterm().add(one2);
			nof2.getSubterm().add(varm2);
			HLAnnotation hlannot1 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot1.setStructure(nof2);
			hlannot1.setText("1'(m)");
			ArcHLAPI mem2eoa = new ArcHLAPI("mem2eoa", new NameHLAPI("Memory2End_Own_Acc"), memory, endOwnAcc, null,
					new HLAnnotationHLAPI(hlannot1), defaultPage);
			// Memory --> Begin_Ext_Acc : 1'(m)
			NumberConstant one3 = IntegersFactory.eINSTANCE.createNumberConstant();
			one3.setValue(1);
			one3.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varm3 = TermsFactory.eINSTANCE.createVariable();
			varm3.setVariableDecl(mDecl.getContainedItem());
			NumberOf nof3 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof3.getSubterm().add(one3);
			nof3.getSubterm().add(varm3);
			HLAnnotation hlannot2 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot2.setStructure(nof3);
			hlannot2.setText("1'(m)");
			ArcHLAPI mem2bea = new ArcHLAPI("mem2bea", new NameHLAPI("Memory2Begin_Ext_Acc"), memory, beginExtAcc,
					null, new HLAnnotationHLAPI(hlannot2), defaultPage);
			// Active --> Begin_Own_Acc : 1'(x)
			NumberConstant one4 = IntegersFactory.eINSTANCE.createNumberConstant();
			one4.setValue(1);
			one4.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx3 = TermsFactory.eINSTANCE.createVariable();
			varx3.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof4 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof4.getSubterm().add(one4);
			nof4.getSubterm().add(varx3);
			HLAnnotation hlannot3 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot3.setStructure(nof4);
			hlannot3.setText("1'(x)");
			ArcHLAPI act2boa = new ArcHLAPI("act2boa", new NameHLAPI("Active2Begin_Own_Acc"), active, beginOwnAcc,
					null, new HLAnnotationHLAPI(hlannot3), defaultPage);
			// Active --> Req_Ext_Acc : 1'(x)
			NumberConstant one5 = IntegersFactory.eINSTANCE.createNumberConstant();
			one5.setValue(1);
			one5.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx4 = TermsFactory.eINSTANCE.createVariable();
			varx4.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof5 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof5.getSubterm().add(one5);
			nof5.getSubterm().add(varx4);
			HLAnnotation hlannot4 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot4.setStructure(nof5);
			hlannot4.setText("1'(x)");
			ArcHLAPI act2rea = new ArcHLAPI("act2rea", new NameHLAPI("Active2Req_Ext_Acc"), active, reqExtAcc, null,
					new HLAnnotationHLAPI(hlannot4), defaultPage);
			// Queue --> Begin_Ext_Acc : 1'(x)
			NumberConstant one6 = IntegersFactory.eINSTANCE.createNumberConstant();
			one6.setValue(1);
			one6.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx5 = TermsFactory.eINSTANCE.createVariable();
			varx5.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof6 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof6.getSubterm().add(one6);
			nof6.getSubterm().add(varx5);
			HLAnnotation hlannot5 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot5.setStructure(nof6);
			hlannot5.setText("1'(x)");
			ArcHLAPI que2bea = new ArcHLAPI("que2bea", new NameHLAPI("Queue2Begin_Ext_Acc"), queue, beginExtAcc, null,
					new HLAnnotationHLAPI(hlannot5), defaultPage);
			// Ext_Mem_Acc --> End_Ext_Acc : 1'(x,y)
			NumberConstant one7 = IntegersFactory.eINSTANCE.createNumberConstant();
			one7.setValue(1);
			one7.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx6 = TermsFactory.eINSTANCE.createVariable();
			varx6.setVariableDecl(xDecl.getContainedItem());
			Variable vary0 = TermsFactory.eINSTANCE.createVariable();
			vary0.setVariableDecl(yDecl.getContainedItem());
			Tuple tuple0 = TermsFactory.eINSTANCE.createTuple();
			tuple0.getSubterm().add(varx6);
			tuple0.getSubterm().add(vary0);
			NumberOf nof7 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof7.getSubterm().add(one7);
			nof7.getSubterm().add(tuple0);
			HLAnnotation hlannot6 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot6.setStructure(nof7);
			hlannot6.setText("1'(x)");
			ArcHLAPI ema2eea = new ArcHLAPI("ema2eea", new NameHLAPI("Ext_Mem_Acc2End_Ext_Acc"), extMemAcc, endExtAcc,
					null, new HLAnnotationHLAPI(hlannot6), defaultPage);
			// Ext_Bus --> Begin_Ext_Acc : 1'(dot)
			NumberConstant one8 = IntegersFactory.eINSTANCE.createNumberConstant();
			one8.setValue(1);
			one8.setType(IntegersFactory.eINSTANCE.createPositive());
			DotConstant dotConstant1 = DotsFactory.eINSTANCE.createDotConstant();
			NumberOf nof8 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof8.getSubterm().add(one8);
			nof8.getSubterm().add(dotConstant1);
			HLAnnotation hlannot7 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot7.setStructure(nof8);
			hlannot7.setText("1'(dot)");
			ArcHLAPI exb2bea = new ArcHLAPI("exb2bea", new NameHLAPI("Ext_Bus2Begin_Ext_Acc"), extBus, beginExtAcc,
					null, new HLAnnotationHLAPI(hlannot7), defaultPage);
			// Begin_Own_Acc --> OwnMemAcc : 1'(x)
			NumberConstant one9 = IntegersFactory.eINSTANCE.createNumberConstant();
			one9.setValue(1);
			one9.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx7 = TermsFactory.eINSTANCE.createVariable();
			varx7.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof9 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof9.getSubterm().add(one9);
			nof9.getSubterm().add(varx7);
			HLAnnotation hlannot8 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot8.setStructure(nof9);
			hlannot8.setText("1'(x)");
			ArcHLAPI boa2oma = new ArcHLAPI("boa2oma", new NameHLAPI("Begin_Own_Acc2OwnMemAcc"), beginOwnAcc,
					ownMemAcc, null, new HLAnnotationHLAPI(hlannot8), defaultPage);
			// End_Own_Acc --> Memory : 1'(m)
			NumberConstant one10 = IntegersFactory.eINSTANCE.createNumberConstant();
			one10.setValue(1);
			one10.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varm4 = TermsFactory.eINSTANCE.createVariable();
			varm4.setVariableDecl(mDecl.getContainedItem());
			NumberOf nof10 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof10.getSubterm().add(one10);
			nof10.getSubterm().add(varm4);
			HLAnnotation hlannot9 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot9.setStructure(nof10);
			hlannot9.setText("1'(m)");
			ArcHLAPI eoa2mem = new ArcHLAPI("eoa2mem", new NameHLAPI("End_Own_Acc2Memory"), endOwnAcc, memory, null,
					new HLAnnotationHLAPI(hlannot9), defaultPage);
			// End_Ext_Acc --> Memory : 1'(y)
			NumberConstant one11 = IntegersFactory.eINSTANCE.createNumberConstant();
			one11.setValue(1);
			one11.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable vary1 = TermsFactory.eINSTANCE.createVariable();
			vary1.setVariableDecl(yDecl.getContainedItem());
			NumberOf nof11 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof11.getSubterm().add(one11);
			nof11.getSubterm().add(vary1);
			HLAnnotation hlannot10 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot10.setStructure(nof11);
			hlannot10.setText("1'(y)");
			ArcHLAPI eea2mem = new ArcHLAPI("eea2mem", new NameHLAPI("End_Ext_Acc2Memory"), endExtAcc, memory, null,
					new HLAnnotationHLAPI(hlannot10), defaultPage);
			// End_Own_Acc --> Active : 1'(x)
			NumberConstant one12 = IntegersFactory.eINSTANCE.createNumberConstant();
			one12.setValue(1);
			one12.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx8 = TermsFactory.eINSTANCE.createVariable();
			varx8.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof12 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof12.getSubterm().add(one12);
			nof12.getSubterm().add(varx8);
			HLAnnotation hlannot11 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot11.setStructure(nof12);
			hlannot11.setText("1'(x)");
			ArcHLAPI eoa2act = new ArcHLAPI("eoa2act", new NameHLAPI("End_Own_Acc2Active"), endOwnAcc, active, null,
					new HLAnnotationHLAPI(hlannot11), defaultPage);
			// End_Ext_Acc --> Active : 1'(x)
			NumberConstant one13 = IntegersFactory.eINSTANCE.createNumberConstant();
			one13.setValue(1);
			one13.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx9 = TermsFactory.eINSTANCE.createVariable();
			varx9.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof13 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof13.getSubterm().add(one13);
			nof13.getSubterm().add(varx9);
			HLAnnotation hlannot12 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot12.setStructure(nof13);
			hlannot12.setText("1'(x)");
			ArcHLAPI eea2act = new ArcHLAPI("eea2act", new NameHLAPI("End_Ext_Acc2Active"), endExtAcc, active, null,
					new HLAnnotationHLAPI(hlannot12), defaultPage);
			// Req_Ext_Acc --> Queue : 1'(x)
			NumberConstant one14 = IntegersFactory.eINSTANCE.createNumberConstant();
			one14.setValue(1);
			one14.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx10 = TermsFactory.eINSTANCE.createVariable();
			varx10.setVariableDecl(xDecl.getContainedItem());
			NumberOf nof14 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof14.getSubterm().add(one14);
			nof14.getSubterm().add(varx10);
			HLAnnotation hlannot13 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot13.setStructure(nof14);
			hlannot13.setText("1'(x)");
			ArcHLAPI rea2que = new ArcHLAPI("rea2que", new NameHLAPI("Req_Ext_Acc2Queue"), reqExtAcc, queue, null,
					new HLAnnotationHLAPI(hlannot13), defaultPage);
			// Begin_Ext_Acc --> Ext_Mem_Acc : 1'(x,m)
			NumberConstant one15 = IntegersFactory.eINSTANCE.createNumberConstant();
			one15.setValue(1);
			one15.setType(IntegersFactory.eINSTANCE.createPositive());
			Variable varx11 = TermsFactory.eINSTANCE.createVariable();
			varx11.setVariableDecl(xDecl.getContainedItem());
			Variable varm5 = TermsFactory.eINSTANCE.createVariable();
			varm5.setVariableDecl(mDecl.getContainedItem());
			Tuple tuple1 = TermsFactory.eINSTANCE.createTuple();
			tuple1.getSubterm().add(varx11);
			tuple1.getSubterm().add(varm5);
			NumberOf nof15 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof15.getSubterm().add(one15);
			nof15.getSubterm().add(tuple1);
			HLAnnotation hlannot14 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot14.setStructure(nof15);
			hlannot14.setText("1'(x,m)");
			ArcHLAPI bea2ema = new ArcHLAPI("bea2ema", new NameHLAPI("Begin_Ext_Acc2Ext_Mem_Acc"), beginExtAcc,
					extMemAcc, null, new HLAnnotationHLAPI(hlannot14), defaultPage);
			// End_Ext_Acc --> Ext_Bus : 1'(dot)
			NumberConstant one16 = IntegersFactory.eINSTANCE.createNumberConstant();
			one16.setValue(1);
			one16.setType(IntegersFactory.eINSTANCE.createPositive());
			DotConstant dotConstant2 = DotsFactory.eINSTANCE.createDotConstant();
			NumberOf nof16 = MultisetsFactory.eINSTANCE.createNumberOf();
			nof16.getSubterm().add(one16);
			nof16.getSubterm().add(dotConstant2);
			HLAnnotation hlannot15 = HlcorestructureFactory.eINSTANCE.createHLAnnotation();
			hlannot15.setStructure(nof16);
			hlannot15.setText("1'(dot)");
			ArcHLAPI eea2exb = new ArcHLAPI("eea2exb", new NameHLAPI("End_Ext_Acc2Ext_Bus"), endExtAcc, extBus, null,
					new HLAnnotationHLAPI(hlannot15), defaultPage);
		} catch (InvalidIDException e) {
			e.printStackTrace();
		} catch (VoidRepositoryException e) {
			e.printStackTrace();
		}
	}
}
